# -*- coding: UTF-8 -*-

#########################################################
# Name: ffmpeg_decoders.py
# Porpose: Dialog to show the available decoders on the FFmpeg
# Compatibility: Python3, wxPython Phoenix
# Author: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2019 Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Rev: Aug.19 2019
#########################################################

# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################

import wx

class FFmpeg_Codecs(wx.Dialog):
    """
    It shows a dialog box with a pretty kind of GUI to view 
    the formats available on FFmpeg
    
    """
    def __init__(self, dict_decoders, OS, type_opt):
        """
        with 'None' not depend from parent:
        wx.Dialog.__init__(self, None, style=wx.DEFAULT_DIALOG_STYLE)
        
        With parent, -1:
        wx.Dialog.__init__(self, parent, -1, style=wx.DEFAULT_DIALOG_STYLE)
        if close videomass also close parent window:
        
        """
        if type_opt == '-encoders':
            cod =  _('CODING ABILITY')
            colctrl = 'ORANGE'
            title = _("Videomass: FFmpeg encoders")
        else:
            cod =  _('DECODING CAPABILITY')
            colctrl = 'SIENNA'
            title = _("Videomass: FFmpeg decoders")
            
        wx.Dialog.__init__(self, None, style=wx.DEFAULT_DIALOG_STYLE)
        notebook_1 = wx.Notebook(self, wx.ID_ANY,)
        notebook_1_pane_1 = wx.Panel(notebook_1, wx.ID_ANY)
        vid = wx.ListCtrl(notebook_1_pane_1, wx.ID_ANY, 
                                    style=wx.LC_REPORT | 
                                    wx.SUNKEN_BORDER
                                    )
        notebook_1_pane_2 = wx.Panel(notebook_1, wx.ID_ANY)
        aud = wx.ListCtrl(notebook_1_pane_2, wx.ID_ANY, 
                                     style=wx.LC_REPORT | 
                                     wx.SUNKEN_BORDER
                                     )
        notebook_1_pane_3 = wx.Panel(notebook_1, wx.ID_ANY)
        sub = wx.ListCtrl(notebook_1_pane_3, wx.ID_ANY, 
                                       style=wx.LC_REPORT | 
                                       wx.SUNKEN_BORDER
                                       )
        stext = wx.StaticText(self, wx.ID_ANY, "")
        #button_help = wx.Button(self, wx.ID_HELP, "")
        button_close = wx.Button(self, wx.ID_CLOSE, "")
        
        #----------------------Properties----------------------#
        self.SetTitle(title)
        vid.SetMinSize((600, 300))
        vid.InsertColumn(0, ('codec'), width=150)
        vid.InsertColumn(1, ('F'), width=40)
        vid.InsertColumn(2, ('S'), width=40)
        vid.InsertColumn(3, ('X'), width=40)
        vid.InsertColumn(4, ('B'), width=40)
        vid.InsertColumn(5, ('D'), width=40)
        vid.InsertColumn(6, _('description'), width=450)
        #vid.SetBackgroundColour(wx.Colour(217, 255, 255))
        aud.SetMinSize((600, 300))
        aud.InsertColumn(0, ('codec'), width=150)
        aud.InsertColumn(1, ('F'), width=40)
        aud.InsertColumn(2, ('S'), width=40)
        aud.InsertColumn(3, ('X'), width=40)
        aud.InsertColumn(4, ('B'), width=40)
        aud.InsertColumn(5, ('D'), width=40)
        aud.InsertColumn(6, _('description'), width=450)
        #aud.SetBackgroundColour(wx.Colour(217, 255, 255))
        sub.SetMinSize((600, 300))
        sub.InsertColumn(0, ('codec'), width=150)
        sub.InsertColumn(1, ('F'), width=40)
        sub.InsertColumn(2, ('S'), width=40)
        sub.InsertColumn(3, ('X'), width=40)
        sub.InsertColumn(4, ('B'), width=40)
        sub.InsertColumn(5, ('D'), width=40)
        sub.InsertColumn(6, _('description'), width=450)
        #sub.SetBackgroundColour(wx.Colour(217, 255, 255))
        
        if OS == 'Darwin':
            vid.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL, wx.NORMAL))
            aud.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL, wx.NORMAL))
            sub.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL, wx.NORMAL))
            stext.SetFont(wx.Font(11, wx.SWISS, wx.ITALIC, wx.NORMAL))
        else:
            vid.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL))
            aud.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL))
            sub.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL))
            stext.SetFont(wx.Font(8, wx.SWISS, wx.ITALIC, wx.NORMAL))
        
        leg = ("F = frame-level multithreading\n"
               "S = slice-level multithreading\n"
               "X = Codec is experimental\n"
               "B = Supports draw_horiz_band\n"
               "D = Supports direct rendering method 1")
        stext.SetLabel(leg)
        
        #----------------------Layout--------------------------#
        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        grid_sizer_1 = wx.FlexGridSizer(2, 1, 0, 0)
        grid_buttons = wx.FlexGridSizer(1, 1, 0, 0)
        sizer_tab3 = wx.BoxSizer(wx.VERTICAL)
        sizer_tab2 = wx.BoxSizer(wx.VERTICAL)
        sizer_tab1 = wx.BoxSizer(wx.VERTICAL)
        sizer_tab1.Add(vid, 1, wx.ALL | wx.EXPAND, 5)
        notebook_1_pane_1.SetSizer(sizer_tab1)
        sizer_tab2.Add(aud, 1, wx.ALL | wx.EXPAND, 5)
        notebook_1_pane_2.SetSizer(sizer_tab2)
        sizer_tab3.Add(sub, 1, wx.ALL | wx.EXPAND, 5)
        notebook_1_pane_3.SetSizer(sizer_tab3)
        notebook_1.AddPage(notebook_1_pane_1, (_("Video")))
        notebook_1.AddPage(notebook_1_pane_2, (_("Audio")))
        notebook_1.AddPage(notebook_1_pane_3, (_("Subtitle")))
        grid_sizer_1.Add(notebook_1, 1, wx.ALL|wx.EXPAND, 5)
        grid_sizer_1.Add(stext, 1, wx.ALL, 5)
        grid_buttons.Add(button_close, 0, wx.ALL, 5)
        sizer_1.Add(grid_sizer_1, 1, wx.EXPAND, 0)
        sizer_1.Add(grid_buttons, flag=wx.ALIGN_RIGHT|wx.RIGHT, border=0)

        self.SetSizerAndFit(sizer_1)
        
        #### populate vid listctrl output:
        index = 0 
        l = dict_decoders['Video']
        if not l:
            print ('No ffmpeg codecs available')
        else:
            vid.InsertItem(index, cod)
            vid.SetItemBackgroundColour(index, colctrl)
            for a in l:
                index+=1
                vid.InsertItem(index, a[6:].split(' ')[1])
                if 'F' in a[1]:
                    vid.SetItem(index, 1, 'YES')
                if 'S' in a[2]:
                    vid.SetItem(index, 2, 'YES')
                if 'X' in a[3]:
                    vid.SetItem(index, 3, 'YES')
                if 'B' in [4]:
                    vid.SetItem(index, 4, 'YES')
                if 'D' in [5]:
                    vid.SetItem(index, 5, 'YES')
                d = " ".join(a.split()).split(None,2)[2]
                if len(d):
                    vid.SetItem(index, 6, d)
                else:
                    vid.SetItem(index, 6, '')
                
        ##### populate aud listctrl output:
        index = 0 
        l = dict_decoders['Audio']
        if not l:
            print ('No ffmpeg codecs available')
        else:
            aud.InsertItem(index, cod)
            aud.SetItemBackgroundColour(index, colctrl)
            for a in l:
                index+=1
                aud.InsertItem(index, a[6:].split(' ')[1])
                if 'F' in a[1]:
                    aud.SetItem(index, 1, 'YES')
                if 'S' in a[2]:
                    aud.SetItem(index, 2, 'YES')
                if 'X' in a[3]:
                    aud.SetItem(index, 3, 'YES')
                if 'B' in [4]:
                    aud.SetItem(index, 4, 'YES')
                if 'D' in [5]:
                    aud.SetItem(index, 5, 'YES')
                d = " ".join(a.split()).split(None,2)[2]
                if len(d):
                    aud.SetItem(index, 6, d)
                else:
                    aud.SetItem(index, 6, '')
        
        ###### populate sub listctrl output:
        index = 0 
        l = dict_decoders['Subtitle']
        if not l:
            print('No ffmpeg codecs available')
        else:
            sub.InsertItem(index, cod)
            sub.SetItemBackgroundColour(index, colctrl)
            for a in l:
                index+=1
                sub.InsertItem(index, a[6:].split(' ')[1])
                if 'F' in a[1]:
                    sub.SetItem(index, 1, 'YES')
                if 'S' in a[2]:
                    sub.SetItem(index, 2, 'YES')
                if 'X' in a[3]:
                    sub.SetItem(index, 3, 'YES')
                if 'B' in [4]:
                    sub.SetItem(index, 4, 'YES')
                if 'D' in [5]:
                    sub.SetItem(index, 5, 'YES')
                d = " ".join(a.split()).split(None,2)[2]
                if len(d):
                    sub.SetItem(index, 6, d)
                else:
                    sub.SetItem(index, 6, '')
                    
        #----------------------Binding (EVT)----------------------#
        self.Bind(wx.EVT_BUTTON, self.on_close, button_close)
        self.Bind(wx.EVT_CLOSE, self.on_close) # controlla la chiusura (x)
        #self.Bind(wx.EVT_BUTTON, self.on_help, button_help)

    #----------------------Event handler (callback)----------------------#
    def on_close(self, event):
        self.Destroy()
        #event.Skip()

    #-------------------------------------------------------------------# 



# -*- coding: UTF-8 -*-

#########################################################
# Name: main_frame.py
# Porpose: top window main frame
# Compatibility: Python3, wxPython Phoenix
# Author: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2019 Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Rev: Dec 28 2018, Sept 10 2019
#########################################################

# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################

import wx
import wx.lib.agw.gradientbutton as GB
import webbrowser
from videomass3.vdms_DIALOGS import dialog_tools
from videomass3.vdms_DIALOGS import settings
from videomass3.vdms_DIALOGS import infoprg
from videomass3.vdms_DIALOGS import while_playing
from videomass3.vdms_DIALOGS import ffmpeg_search
from videomass3.vdms_PANELS import dragNdrop, presets_mng_panel
from videomass3.vdms_PANELS import video_conv, audio_conv
from videomass3.vdms_IO import IO_tools

# set widget colours in some case with html rappresentetion:
azure = '#d9ffff' # rgb form (wx.Colour(217,255,255))
yellow = '#a29500'
red = '#ea312d'
orange = '#f28924'
greenolive = '#6aaf23'

########################################################################
class MainFrame(wx.Frame):
    """
    This is the main frame top window for panels implementation.
    Currently it host four panels, three of which are instantiated 
    in the init constructor. The fourth panel is instantiated in an 
    appropriate instance method. (see switch_Process method doc strings)
    """
    def __init__(self, setui, ffmpeg_link,
                 ffprobe_link, ffplay_link, 
                 pathicons):
        """
        NOTE: 'SRCpath' is a current work directory of Videomass 
               program. How it can be localized depend if Videomass is 
               run as portable program or installated program.
        """

        self.videomass_icon = pathicons[0]
        self.icon_presets = pathicons[1]
        self.icon_switchvideomass = pathicons[2]
        self.icon_process = pathicons[3]
        self.icon_help = pathicons[4]
        self.icon_headphones = pathicons[5]
        self.icon_import = pathicons[6]
        barC = setui[4][13].split(',') 
        barColor = wx.Colour(int(barC[0]),int(barC[1]),int(barC[2])) 
        # toolbar panel colour
        bBtnC = setui[4][14].split(',')
        self.bBtnC = wx.Colour(int(bBtnC[0]),int(bBtnC[1]),int(bBtnC[2]))
        # toolbar buttons colour
        
        #self.helping = setui[5]# path contestual help for helping:
        self.OS = setui[0]# ID of the operative system:
        SRCpath = setui[1]# share dir (are where the origin files?):
        self.PATHconf = setui[6]
        self.WORKdir = setui[7]
        self.DIRconf = setui[8]
        
        #---------------------------#
        self.threads = setui[4][2]#ffmpeg option, set the cpu threads
        self.cpu_used = setui[4][3]
        self.ffplay_loglevel = setui[4][4]
        self.ffmpeg_loglevel = setui[4][5]
        self.loglevel_batch = ''#setui[4][7]# marks as batch process
        self.ffmpeg_check = setui[4][6]
        self.ffprobe_check = setui[4][8]
        self.ffplay_check = setui[4][10]
        self.ffmpeg_link = ffmpeg_link
        self.ffprobe_link = ffprobe_link
        self.ffplay_link = ffplay_link
        self.iconset = setui[4][12]
        #-------------------------------#
        self.import_clicked = ''#when clicking on item in list control self-set 
        self.post_process = []# post-pocess set first file for play/metadata
        self.file_sources = []# list of items in list control
        self.file_destin = ''# path name for file saved destination
        self.panelshown = '' # gives current (previusly) panel shown
        self.time_seq = ''
        self.duration = []

        wx.Frame.__init__(self, None, -1, style=wx.DEFAULT_FRAME_STYLE)
        #----------- panel toolbar buttons
        self.btnpanel = wx.Panel(self, wx.ID_ANY, style=wx.TAB_TRAVERSAL)
        
        infoIbmp = wx.Bitmap(pathicons[7], wx.BITMAP_TYPE_ANY)
        previewbmp = wx.Bitmap(pathicons[8], wx.BITMAP_TYPE_ANY)
        cutbmp = wx.Bitmap(pathicons[9], wx.BITMAP_TYPE_ANY)
        saveprfbmp = wx.Bitmap(pathicons[12], wx.BITMAP_TYPE_ANY)
        newprfbmp = wx.Bitmap(pathicons[13], wx.BITMAP_TYPE_ANY)
        delprfbmp = wx.Bitmap(pathicons[14], wx.BITMAP_TYPE_ANY)
        editprfbmp = wx.Bitmap(pathicons[15], wx.BITMAP_TYPE_ANY)
        
        
        
        self.btn_metaI = GB.GradientButton(self.btnpanel,
                                           size=(-1,25),
                                           bitmap=infoIbmp, 
                                           label=_("Show Metadata"))
        self.btn_metaI.SetBaseColours(startcolour=wx.Colour(158,201,232), 
                                      foregroundcolour=wx.Colour(28,28,28))
        self.btn_metaI.SetBottomEndColour(self.bBtnC)
        self.btn_metaI.SetBottomStartColour(self.bBtnC)
        self.btn_metaI.SetTopStartColour(self.bBtnC)
        self.btn_metaI.SetTopEndColour(self.bBtnC)
        
        self.btn_playO = GB.GradientButton(self.btnpanel,
                                           size=(-1,25),
                                           bitmap=previewbmp, 
                                           label=_("Preview"))
        self.btn_playO.SetBaseColours(startcolour=wx.Colour(158,201,232), 
                                      foregroundcolour=wx.Colour(28,28,28))
        self.btn_playO.SetBottomEndColour(self.bBtnC)
        self.btn_playO.SetBottomStartColour(self.bBtnC)
        self.btn_playO.SetTopStartColour(self.bBtnC)
        self.btn_playO.SetTopEndColour(self.bBtnC)
        
        self.btn_duration = GB.GradientButton(self.btnpanel,
                                              size=(-1,25),
                                              bitmap=cutbmp, 
                                              label=_("Duration"))
        self.btn_duration.SetBaseColours(startcolour=wx.Colour(158,201,232), 
                                    foregroundcolour=wx.Colour(28,28,28))
        self.btn_duration.SetBottomEndColour(self.bBtnC)
        self.btn_duration.SetBottomStartColour(self.bBtnC)
        self.btn_duration.SetTopStartColour(self.bBtnC)
        self.btn_duration.SetTopEndColour(self.bBtnC)
        
        self.btn_saveprf = GB.GradientButton(self.btnpanel,
                                              size=(-1,25),
                                              bitmap=saveprfbmp, 
                                              label=_("Save As Profile"))
        self.btn_saveprf.SetBaseColours(startcolour=wx.Colour(158,201,232), 
                                    foregroundcolour=wx.Colour(28,28,28))
        self.btn_saveprf.SetBottomEndColour(self.bBtnC)
        self.btn_saveprf.SetBottomStartColour(self.bBtnC)
        self.btn_saveprf.SetTopStartColour(self.bBtnC)
        self.btn_saveprf.SetTopEndColour(self.bBtnC)
        
        self.btn_newprf = GB.GradientButton(self.btnpanel,
                                              size=(-1,25),
                                              bitmap=newprfbmp, 
                                              label=_("New.."))
        self.btn_newprf.SetBaseColours(startcolour=wx.Colour(158,201,232), 
                                    foregroundcolour=wx.Colour(28,28,28))
        self.btn_newprf.SetBottomEndColour(self.bBtnC)
        self.btn_newprf.SetBottomStartColour(self.bBtnC)
        self.btn_newprf.SetTopStartColour(self.bBtnC)
        self.btn_newprf.SetTopEndColour(self.bBtnC)
        
        self.btn_delprf = GB.GradientButton(self.btnpanel,
                                              size=(-1,25),
                                              bitmap=delprfbmp, 
                                              label=_("Delete.."))
        self.btn_delprf.SetBaseColours(startcolour=wx.Colour(158,201,232), 
                                    foregroundcolour=wx.Colour(28,28,28))
        self.btn_delprf.SetBottomEndColour(self.bBtnC)
        self.btn_delprf.SetBottomStartColour(self.bBtnC)
        self.btn_delprf.SetTopStartColour(self.bBtnC)
        self.btn_delprf.SetTopEndColour(self.bBtnC)
        
        self.btn_editprf = GB.GradientButton(self.btnpanel,
                                              size=(-1,25),
                                              bitmap=editprfbmp, 
                                              label=_("Edit.."))
        self.btn_editprf.SetBaseColours(startcolour=wx.Colour(158,201,232), 
                                    foregroundcolour=wx.Colour(28,28,28))
        self.btn_editprf.SetBottomEndColour(self.bBtnC)
        self.btn_editprf.SetBottomStartColour(self.bBtnC)
        self.btn_editprf.SetTopStartColour(self.bBtnC)
        self.btn_editprf.SetTopEndColour(self.bBtnC)

        self.btnpanel.SetBackgroundColour(barColor)
        #self.btnpanel.SetBackgroundColour(wx.Colour(205, 235, 222))
        #---------- others panel instances:
        self.PrstsPanel = presets_mng_panel.PresetsPanel(self, SRCpath, 
                                                         self.DIRconf, 
                                                         self.WORKdir, 
                                                         self.threads, 
                                                         self.cpu_used,
                                                         self.ffmpeg_loglevel, 
                                                         self.ffmpeg_link, 
                                                         self.OS,
                                                         )
        self.VconvPanel = video_conv.Video_Conv(self, self.ffmpeg_link,
                                                self.ffplay_link,
                                                self.threads, 
                                                self.cpu_used,
                                                self.ffmpeg_loglevel,
                                                self.ffplay_loglevel,
                                                self.OS,
                                                pathicons[10],# icon playfilters
                                                pathicons[11],# icon resetfilters
                                                pathicons[16],# icon resize
                                                pathicons[17],# icon crop
                                                pathicons[18],# icon rotate
                                                pathicons[19],# icon deinterlace
                                                pathicons[20],# icon ic_denoiser
                                                pathicons[21],# icon analyzes
                                                pathicons[22],# icon settings
                                                )
        self.AconvPanel = audio_conv.Audio_Conv(self, self.ffmpeg_link, 
                                                self.threads,
                                                self.cpu_used,
                                                self.ffmpeg_loglevel, 
                                                self.ffprobe_link,
                                                self.OS,
                                                pathicons[21],# icon analyzes
                                                pathicons[22],# icon settings
                                                )

        self.DnD = dragNdrop.DnDPanel(self, self.ffprobe_link) # dragNdrop panel
        self.PrstsPanel.Hide()
        self.VconvPanel.Hide()
        self.AconvPanel.Hide()
        # Layout toolbar buttons:
        self.DnDsizer = wx.BoxSizer(wx.VERTICAL) # sizer base global
        grid_pan = wx.FlexGridSizer(1, 7, 0, 0)
        grid_pan.Add(self.btn_metaI, 0, wx.CENTER|wx.ALL, 5)
        grid_pan.Add(self.btn_playO, 0, wx.CENTER|wx.ALL, 5)
        grid_pan.Add(self.btn_duration, 0, wx.CENTER|wx.ALL, 5)
        grid_pan.Add(self.btn_saveprf, 0, wx.CENTER|wx.ALL, 5)
        grid_pan.Add(self.btn_newprf, 0, wx.CENTER|wx.ALL, 5)
        grid_pan.Add(self.btn_delprf, 0, wx.CENTER|wx.ALL, 5)
        grid_pan.Add(self.btn_editprf, 0, wx.CENTER|wx.ALL, 5)
        self.btnpanel.SetSizer(grid_pan) # set panel
        self.DnDsizer.Add(self.btnpanel, 0, wx.EXPAND, 0)
        # Layout externals panels:
        self.DnDsizer.Add(self.DnD, 1, wx.EXPAND|wx.ALL, 0)
        self.DnDsizer.Add(self.PrstsPanel, 1, wx.EXPAND|wx.ALL, 0)
        self.DnDsizer.Add(self.VconvPanel, 1, wx.EXPAND|wx.ALL, 0)
        self.DnDsizer.Add(self.AconvPanel, 1, wx.EXPAND|wx.ALL, 0)
        
        #----------------------Set Properties----------------------#
        self.SetTitle("Videomass")
        self.btn_playO.Hide()
        icon = wx.Icon()
        icon.CopyFromBitmap(wx.Bitmap(self.videomass_icon, wx.BITMAP_TYPE_ANY))
        self.SetIcon(icon)
        if self.OS == 'Darwin':
            self.SetSize((920, 435))
        elif self.OS == 'Windows':
            self.SetSize((900, 530))
        else:
            self.SetSize((1000, 600))
        #self.Centre()
        #self.CentreOnScreen() # se lo usi, usa CentreOnScreen anziche Centre
        self.SetSizer(self.DnDsizer)
        self.Layout()
        
        # Tooltips:
        self.btn_duration.SetToolTip(_('Set a global timeline to apply to '
                                       'any media file with duration.'
                                           ))
        self.btn_metaI.SetToolTip(_("Show information about the metadata "
                                        "of the selected imported file." 
                                        ))
        self.btn_playO.SetToolTip(_("Play the file exported in the "
                                    "last encoding."
                                        ))
        self.btn_saveprf.SetToolTip(_("Save as profile with the current "
                                        "settings of this panel."
                                        ))
        self.btn_newprf.SetToolTip(_("Create a new profile from yourself "
                                        "and save it in the selected preset."
                                        ))
        self.btn_delprf.SetToolTip(_("Delete the selected profile."
                                        ))
        self.btn_editprf.SetToolTip(_("Edit the selected profile."
                                          ))
        # menu bar
        self.videomass_menu_bar()
        ## tool bar main
        self.videomass_tool_bar()
        self.Setup_items_bar()
        # status bar
        self.sb = self.CreateStatusBar(1)
        
        #---------------------- Binding (EVT) ----------------------#
        self.DnD.ckbx_dir.Bind(wx.EVT_CHECKBOX, self.onCheckBox)
        self.DnD.btn_save.Bind(wx.EVT_BUTTON, self.onCustomSave)
        self.Bind(wx.EVT_BUTTON, self.Cut_range, self.btn_duration)
        self.Bind(wx.EVT_BUTTON, self.Saveprofile, self.btn_saveprf)
        self.Bind(wx.EVT_BUTTON, self.Newprofile, self.btn_newprf)
        self.Bind(wx.EVT_BUTTON, self.Delprofile, self.btn_delprf)
        self.Bind(wx.EVT_BUTTON, self.Editprofile, self.btn_editprf)
        self.Bind(wx.EVT_BUTTON, self.ImportInfo, self.btn_metaI)
        self.Bind(wx.EVT_BUTTON, self.ExportPlay, self.btn_playO)
        #self.Bind(wx.EVT_SHOW, self.panelShown)
        #self.DnDPanel.fileListCtrl.Bind(wx.EVT_LIST_INSERT_ITEM, self.new_isertion)
        self.Bind(wx.EVT_CLOSE, self.on_close) # controlla la chiusura (x)
        #-----------------------------------------------------------#
        self.statusbar_msg(_('Add Files'),None)#set default statusmsg
        
    #-------------------Status bar popolate--------------------#
    def statusbar_msg(self, msg, color):
        """
        set the status-bar with messages and color types
        """
        if color == None:
            self.sb.SetBackgroundColour(wx.NullColour)
        else:
            self.sb.SetBackgroundColour(color)
        self.sb.SetStatusText(msg)
        self.sb.Refresh()
    
    #---------------------- Used Methods ----------------------#
    # The Used Methods are called from the dragNdrop panel
    def Disable_ToolBtn(self):
        """
        Start with default or by deleting listcontrol from dragNdrop panel
        """
        self.toolbar.EnableTool(wx.ID_FILE3, False)
        self.toolbar.EnableTool(wx.ID_FILE5, False)
        self.toolbar.EnableTool(wx.ID_FILE6, False)
        self.toolbar.EnableTool(wx.ID_FILE7, False)
        self.toolbar.EnableTool(wx.ID_OK, False)
    #------------------------------------------------------------------#
    def Enable_ToolBtn(self):
        """
        Enable preset manager, video converter and 
        audio converter buttons
        """
        self.toolbar.EnableTool(wx.ID_FILE5, True)
        self.toolbar.EnableTool(wx.ID_FILE6, True)
        self.toolbar.EnableTool(wx.ID_FILE7, True)
    #------------------------------------------------------------------#
    def Setup_items_bar(self):
        """
        When switch between panels, disable/enable some item on menu bar 
        and tool bar. Also at the program start, the DnDPanel is show and 
        then take its setting
        """
        if self.DnD.IsShown():
            self.file_open.Enable(False), self.saveme.Enable(False), 
            self.restore.Enable(False), self.default.Enable(False), 
            self.default_all.Enable(False), self.refresh.Enable(False), 
            self.btn_newprf.Hide(), self.btn_saveprf.Hide(), 
            self.btn_delprf.Hide(), self.btn_editprf.Hide(),
            
        elif self.PrstsPanel.IsShown():
            self.file_open.Enable(True), self.saveme.Enable(True), 
            self.restore.Enable(True), self.default.Enable(True), 
            self.default_all.Enable(True), self.refresh.Enable(True), 
            self.btn_newprf.Show(), self.btn_saveprf.Hide(), 
            self.btn_delprf.Show(), self.btn_editprf.Show(), 
            self.toolbar.EnableTool(wx.ID_FILE3, True)
            self.toolbar.EnableTool(wx.ID_FILE6, True)
            self.toolbar.EnableTool(wx.ID_FILE7, True)
            self.toolbar.EnableTool(wx.ID_FILE5, False)
            self.toolbar.EnableTool(wx.ID_OK, True)
            self.Layout()
            
        elif self.VconvPanel.IsShown():
            self.file_open.Enable(True), self.saveme.Enable(False),
            self.restore.Enable(False), self.default.Enable(False), 
            self.default_all.Enable(False), self.refresh.Enable(False), 
            self.btn_newprf.Hide(), self.btn_saveprf.Show(), 
            self.btn_delprf.Hide(), self.btn_editprf.Hide(), 
            self.toolbar.EnableTool(wx.ID_FILE3, True)
            self.toolbar.EnableTool(wx.ID_FILE5, True)
            self.toolbar.EnableTool(wx.ID_FILE7, True)
            self.toolbar.EnableTool(wx.ID_FILE6, False)
            self.toolbar.EnableTool(wx.ID_OK, True)
            self.Layout()
            
        elif self.AconvPanel.IsShown():
            self.file_open.Enable(True), self.saveme.Enable(False),
            self.restore.Enable(False), self.default.Enable(False), 
            self.default_all.Enable(False), self.refresh.Enable(False), 
            self.btn_newprf.Hide(), self.btn_saveprf.Show(), 
            self.btn_delprf.Hide(), self.btn_editprf.Hide(), 
            self.toolbar.EnableTool(wx.ID_FILE3, True)
            self.toolbar.EnableTool(wx.ID_FILE5, True)
            self.toolbar.EnableTool(wx.ID_FILE6, True)
            self.toolbar.EnableTool(wx.ID_FILE7, False)
            self.toolbar.EnableTool(wx.ID_OK, True)
            self.Layout()
            
        elif self.ProcessPanel.IsShown():
            self.file_open.Enable(False), self.saveme.Enable(False), 
            self.restore.Enable(False), self.default.Enable(False), 
            self.default_all.Enable(False), self.refresh.Enable(False), 
            #Disable all top menu bar :
            [self.menuBar.EnableTop(x, False) for x in range(0,3)]
            #Disable the tool bar
            self.toolbar.EnableTool(wx.ID_FILE3, False)
            self.toolbar.EnableTool(wx.ID_FILE5, False)
            self.toolbar.EnableTool(wx.ID_FILE6, False)
            self.toolbar.EnableTool(wx.ID_FILE7, False)
            self.toolbar.EnableTool(wx.ID_OK, False)

    #------------------------------------------------------------------#
    def importClicked_enable(self, path):
        """
        when click with the mouse on a control list item, 
        enable Metadata Info and file reproduction menu
        """
        self.btn_metaI.SetBottomEndColour(wx.Colour(0, 240, 0))
        self.import_clicked = path# used for play and metadata
        
    #------------------------------------------------------------------#
    def importClicked_disable(self):
        """
        Disable streams imported menu
        """
        self.btn_metaI.SetBottomEndColour(self.bBtnC)
        self.import_clicked = ''
        
    #------------------------------------------------------------------#
    def postExported_enable(self):
        """
        Enable menu Streams items for output play and metadata
        info
        """
        if not self.btn_playO.IsShown():
            self.btn_playO.Show()
            self.Layout()
        self.btn_playO.SetBottomEndColour(wx.Colour(0, 240, 0))

    #---------------------- Event handler (callback) ------------------#
    # This series of events are interceptions of the dragNdrop panel
    #-------------------------------- Options ----------------------------#
    def Cut_range(self, event):
        """
        Call dialog for Set a time selection cutting on all imported
        media. The values persist so that they are not reset.
        """
        data = ''

        dial = dialog_tools.Cut_Range(self, self.time_seq)
        retcode = dial.ShowModal()
        if retcode == wx.ID_OK:
            data = dial.GetValue()
            if data == '-ss 00:00:00 -t 00:00:00':
                data = ''
                self.btn_duration.SetBottomEndColour(self.bBtnC)
            else:
                self.btn_duration.SetBottomEndColour(wx.Colour(0, 240, 0))
            self.time_seq = data
        else:
            dial.Destroy()
            return
    #------------------------------ Menu  Streams -----------------------#
    def ImportPlay(self):
        """
        Redirect input file clicked at stream_play for playback feature.
        This feature is available by context menu in drag n drop panel only.
        """
        filepath = self.import_clicked
        IO_tools.stream_play(filepath, 
                             self.time_seq, 
                             self.ffplay_link,
                             '', # parameters
                             self.ffplay_loglevel,
                             )
    #------------------------------------------------------------------#
    def ImportInfo(self, event):
        """
        Redirect input file clicked at stream_info for metadata display
        """
        filepath = self.import_clicked
        
        if not filepath:
            wx.MessageBox(_('To view the information from multimedia '
                            'streams, select one of the files in the '
                            '"Add files" panel'), 
                            'Videomass', wx.ICON_INFORMATION, self)
            return
        title = _('Videomass: media streams')
        IO_tools.stream_info(title, 
                             filepath, 
                             self.ffprobe_link,
                             )
    #------------------------------------------------------------------#
    def ExportPlay(self, event):
        """
        Playback functionality for exported files, useful for result 
        testing. The first one exported of the list will be reproduced.
        """
        if not self.post_process:
            wx.MessageBox(_("No files exported yet"), 
                          'Videomass', wx.ICON_INFORMATION, self)
            return

        IO_tools.stream_play(self.post_process,
                             '', # time_seq is useless for the exported file
                             self.ffplay_link, 
                             '', # no others parameters are needed
                             self.ffplay_loglevel,
                             )
    #------------------------------------------------------------------#
    def Saveprofile(self, event):
        """
        Store new profile with same current settings of the panel shown 
        (Video Conversion & Audio Conversion)..
        Every modification on the panel shown will be reported when saving 
        a new profile.
        """
        if self.VconvPanel.IsShown():
            self.VconvPanel.Addprof()
        elif self.AconvPanel.IsShown():
            self.AconvPanel.Addprof()
        else:
            print ('Videomass: Error, no panels shown')
    #------------------------------------------------------------------#
    def Newprofile(self, event):
        """
        Store new profile in the selected preset of the presets manager
        panel. The list is reloaded automatically after pressed ok button 
        in the dialog for update view.
        """
        if self.PrstsPanel.IsShown():
            self.PrstsPanel.Addprof()
        else:
            print ('Videomass: Error, no presets manager panel shown')
    #------------------------------------------------------------------#
    def Delprofile(self, event):
        """
        Delete the selected preset of the presets manager
        panel.
        """
        if self.PrstsPanel.IsShown():
            self.PrstsPanel.Delprof()
    #------------------------------------------------------------------#
    def Editprofile(self, event):
        """
        Edit selected item in the list control of the presets manager
        panel. The list is reloaded automatically after pressed ok button 
        in the dialog.
        """
        if self.PrstsPanel.IsShown():
            self.PrstsPanel.Editprof(self)
    #-----------------------------------------------------------------#
    def onCheckBox(self, event):
        """
        Intercept the Checkbox event in the dragNdrop panel
        and set same file sources destination path
        """
        self.DnD.same_filedest()
        self.file_destin = self.DnD.file_dest
    #------------------------------------------------------------------#
    def onCustomSave(self, event):
        """
        Intercept the button 'save' event in the dragNdrop panel
        and set file custom destination path
        """
        self.DnD.on_custom_save()
        self.file_destin = self.DnD.file_dest
    #------------------------------------------------------------------#
    def on_close(self, event):
        """
        destroy the videomass.
        """
        self.Destroy()
    #------------------------------------------------------------------#

############################### BUILD THE MENU BAR  ########################
    def videomass_menu_bar(self):
        """
        Make a menu bar. Per usare la disabilitazione di un menu item devi
        prima settare l'attributo self sull'item interessato - poi lo gestisci
        con self.item.Enable(False) per disabilitare o (True) per abilitare.
        Se vuoi disabilitare l'intero top di items fai per esempio:
        self.menuBar.EnableTop(6, False) per disabilitare la voce Help.
        """
        self.menuBar = wx.MenuBar()
        
        ####----------------------- file
        fileButton = wx.Menu()
        
        self.file_open = fileButton.Append(wx.ID_OPEN, _("Add File.. "), 
                        _("Files import with drag and drop"))
        self.file_save = fileButton.Append(wx.ID_SAVE, _("Choose a Destination " 
                                                        "folder.."), 
                        _("Choice a folder where save processed files"))
        fileButton.AppendSeparator()
        self.saveme = fileButton.Append(wx.ID_REVERT_TO_SAVED,
                                 _("Save the current preset as separated file"),
                       _("Make a back-up of the selected preset on combobox"
                                    ))
        self.restore = fileButton.Append(wx.ID_REPLACE, _("Restore a previously "
                                                "saved preset"), 
                _("Replace the selected preset with other saved custom preset.")
                                                )
        self.default = fileButton.Append(wx.ID_ANY, _("Reset the current preset "),
                            _("Replace the selected preset with default values.")
                                                )
        fileButton.AppendSeparator()
        
        self.default_all = fileButton.Append(wx.ID_UNDO, _("Reset all presets "),
                         _("Revert all presets to default values")
                                                )
        
        fileButton.AppendSeparator()
        self.refresh = fileButton.Append(wx.ID_REFRESH, _("Reload presets list"), 
                                           _("..Sometimes it can be useful"))
        fileButton.AppendSeparator()
        exitItem = fileButton.Append(wx.ID_EXIT, _("Exit"), _("Close Videomass"))
        self.menuBar.Append(fileButton,"&File")
        
        ####------------------ tools button
        toolsButton = wx.Menu()
        
        playing = toolsButton.Append( wx.ID_ANY, _("While playing"), 
        _("Show dialog box with keyboard shortcuts useful during playback"))
        toolsButton.AppendSeparator()
        checkconf = toolsButton.Append( wx.ID_ANY, _("FFmpeg specifications"), 
                            _("Shows the configuration features of FFmpeg"))
        toolsButton.AppendSeparator()
        ckformats = toolsButton.Append( wx.ID_ANY, _("FFmpeg file formats"),
                                _("Shows file formats available on FFmpeg"))
        toolsButton.AppendSeparator()
        ckcoders = toolsButton.Append( wx.ID_ANY, _("FFmpeg encoders"),
                                _("Shows available encoders on FFmpeg"))
        ckdecoders = toolsButton.Append( wx.ID_ANY, _("FFmpeg decoders"),
                                _("Shows available decoders on FFmpeg"))
        toolsButton.AppendSeparator()
        searchtopic = toolsButton.Append( wx.ID_ANY, _("FFmpeg search topics"),
                        _("Show a dialog box to help you find FFmpeg topics"))
        toolsButton.AppendSeparator()
        openlogdir = toolsButton.Append( wx.ID_ANY, _("Log directory"),
                            _("Opens the Videomass log directory if it exists"))
        openconfdir = toolsButton.Append( wx.ID_ANY,
                                         _("Configuration directory"),
                            _("Opens the Videomass configuration directory"))
        self.menuBar.Append(toolsButton,_("&Tools"))
        
        ####------------------ setup button
        setupButton = wx.Menu()

        self.showtoolbar = setupButton.Append(wx.ID_ANY, _("Show Tool Bar"), 
                                       _("Show tool bar view"), wx.ITEM_CHECK)
        setupButton.Check(self.showtoolbar.GetId(), True)
        
        self.showpanelbar = setupButton.Append(wx.ID_ANY, _("Show Buttons Bar"), 
                                _("Show or hide buttons bar view"), wx.ITEM_CHECK)
        setupButton.Check(self.showpanelbar.GetId(), True)

        setupButton.AppendSeparator()
        setupItem = setupButton.Append(wx.ID_PREFERENCES, _("Setup"), 
                                       _("General Settings"))
        
        self.menuBar.Append(setupButton,_("&Preferences"))
        
        ####------------------ help buton
        helpButton = wx.Menu()
        helpItem = helpButton.Append( wx.ID_HELP, _("User Guide"), "")
        wikiItem = helpButton.Append( wx.ID_ANY, _("Wiki"), "")
        issueItem = helpButton.Append( wx.ID_ANY, _("Issue tracker"), "")
        helpButton.AppendSeparator()
        DonationItem = helpButton.Append( wx.ID_ANY, _("Donation"), "")
        helpButton.AppendSeparator()
        docFFmpeg = helpButton.Append(wx.ID_ANY, _("FFmpeg documentation"), "")
        helpButton.AppendSeparator()
        checkItem = helpButton.Append(wx.ID_ANY, _("Check new releases"), "")
        infoItem = helpButton.Append(wx.ID_ABOUT, _("About Videomass"), "")
        self.menuBar.Append(helpButton, _("&Help"))

        self.SetMenuBar(self.menuBar)
        
        #-----------------------Binding menu bar-------------------------#
        #----FILE----
        self.Bind(wx.EVT_MENU, self.File_Open, self.file_open)
        self.Bind(wx.EVT_MENU, self.File_Save, self.file_save)
        self.Bind(wx.EVT_MENU, self.Saveme, self.saveme)
        self.Bind(wx.EVT_MENU, self.Restore, self.restore)
        self.Bind(wx.EVT_MENU, self.Default, self.default)
        self.Bind(wx.EVT_MENU, self.Default_all, self.default_all)
        self.Bind(wx.EVT_MENU, self.Refresh, self.refresh)
        self.Bind(wx.EVT_MENU, self.Quiet, exitItem)
        #----TOOLS----
        self.Bind(wx.EVT_MENU, self.durinPlayng, playing)
        self.Bind(wx.EVT_MENU, self.Check_conf, checkconf)
        self.Bind(wx.EVT_MENU, self.Check_formats, ckformats)
        self.Bind(wx.EVT_MENU, self.Check_enc, ckcoders)
        self.Bind(wx.EVT_MENU, self.Check_dec, ckdecoders)
        self.Bind(wx.EVT_MENU, self.Search_topic, searchtopic)
        self.Bind(wx.EVT_MENU, self.Openlog, openlogdir)
        self.Bind(wx.EVT_MENU, self.Openconf, openconfdir)
        
        #----SETUP----
        self.Bind(wx.EVT_MENU, self.Show_toolbar, self.showtoolbar)
        self.Bind(wx.EVT_MENU, self.Show_panelbar, self.showpanelbar)
        self.Bind(wx.EVT_MENU, self.Setup, setupItem)
        #----HELP----
        self.Bind(wx.EVT_MENU, self.Helpme, helpItem)
        self.Bind(wx.EVT_MENU, self.Wiki, wikiItem)
        self.Bind(wx.EVT_MENU, self.Issues, issueItem)
        self.Bind(wx.EVT_MENU, self.Donation, DonationItem)
        self.Bind(wx.EVT_MENU, self.DocFFmpeg, docFFmpeg)
        self.Bind(wx.EVT_MENU, self.CheckNewReleases, checkItem)
        self.Bind(wx.EVT_MENU, self.Info, infoItem)
        
    #-------------------Menu Bar Event handler (callback)----------------------#
    
    #-------------------------- Menu  File -----------------------------#
    def File_Open(self, event):
        """
        Open choice dialog input
        """
        self.File_import(self)
    #--------------------------------------------------#
    def File_Save(self, event):
        """
        Open choice dialog output
        """
        self.DnD.on_custom_save()
    #--------------------------------------------------#
    def Saveme(self, event):
        """
        call method for save a single file copy of preset.
        """
        if self.PrstsPanel.IsShown():
            self.PrstsPanel.Saveme()
    #--------------------------------------------------#
    def Restore(self, event):
        """
        call restore a single preset file in the path presets of the program
        """
        if self.PrstsPanel.IsShown():
            self.PrstsPanel.Restore()
    #--------------------------------------------------#
    def Default(self, event):
        """
        call copy the single original preset file into the configuration
        folder. This replace new personal changes make at profile.
        """ 
        if self.PrstsPanel.IsShown():
            self.PrstsPanel.Default()
    #--------------------------------------------------#
    def Default_all(self, event):
        """
        call restore all preset files in the path presets of the program
        """
        if self.PrstsPanel.IsShown():
            self.PrstsPanel.Default_all()
    #--------------------------------------------------#
    def Refresh(self, event):
        """ 
        call Pass to reset_list function for re-charging list
        """
        if self.PrstsPanel.IsShown():
            self.PrstsPanel.Refresh()
    #--------------------------------------------------#
    def Quiet(self, event):
        """
        destroy the videomass.
        """
        self.Destroy()
                
    #------------------------ Menu  Preferences -------------------------#
    def Show_toolbar(self, event):
        """
        Show the tool bar and disable Show toolbar menu item
        """
        if self.showtoolbar.IsChecked():
            self.toolbar.Show()
        else:
            self.toolbar.Hide()
            
        self.Layout()
    #--------------------------------------------------------------------#
    def Show_panelbar(self, event):
        """
        Show or Hide the buttons bar
        """
        if self.showpanelbar.IsChecked():
            self.btnpanel.Show()
        else:
            self.btnpanel.Hide()
            
        self.Layout()
    #------------------------------------------------------------------#
    def Setup(self, event):
        """
        Call the module setup for setting preferences
        """
        #self.parent.Setup(self)
        setup_dlg = settings.Setup(self, self.threads, self.cpu_used,
                                   self.ffplay_loglevel, self.ffmpeg_loglevel,
                                   self.ffmpeg_link, self.ffmpeg_check,
                                   self.ffprobe_link, self.ffprobe_check, 
                                   self.ffplay_link, self.ffplay_check, 
                                   self.OS, self.iconset, self.PATHconf,
                                   self.WORKdir
                                     )
        setup_dlg.ShowModal()
    
    #--------------------------- Menu Tools ---------------------------#
    def durinPlayng(self, event):
        """
        show dialog with shortcuts keyboard for FFplay
        """
        dlg = while_playing.While_Playing(self.OS)
        dlg.Show()
        
    #------------------------------------------------------------------#
    def Check_conf(self, event):
        """
        Call IO_tools.test_conf
        
        """
        IO_tools.test_conf(self.ffmpeg_link, self.ffprobe_link, 
                                 self.ffplay_link
                                 )
    #------------------------------------------------------------------#
    def Check_formats(self, event):
        """
        IO_tools.test_formats
        
        """
        IO_tools.test_formats(self.ffmpeg_link)
    #------------------------------------------------------------------#
    def Check_enc(self, event):
        """
        IO_tools.test_encoders
        
        """
        IO_tools.test_codecs(self.ffmpeg_link, '-encoders')
    #------------------------------------------------------------------#
    def Check_dec(self, event):
        """
        IO_tools.test_encoders
        
        """
        IO_tools.test_codecs(self.ffmpeg_link, '-decoders')
    #------------------------------------------------------------------#
    def Search_topic(self, event):
        """
        Show a dialog box to help you find FFmpeg topics
        
        """
        dlg = ffmpeg_search.FFmpeg_Search(self.ffmpeg_link, self.OS)
        dlg.Show()
    #------------------------------------------------------------------#
    def Openlog(self, event):
        """
        Open the log diretctory with file manager
        
        """
        IO_tools.openpath('openlog')
    #------------------------------------------------------------------#
    def Openconf(self, event):
        """
        Open the configuration folder with file manager
        
        """
        IO_tools.openpath('dirconf')
        
    #---------------------------- Menu Edit ----------------------------#
    def Helpme(self, event):
        """Online User guide"""
        page = 'https://jeanslack.github.io/Videomass/videomass_use.html'
        webbrowser.open(page)

    #------------------------------------------------------------------#
    def Wiki(self, event):
        """Wiki page """
        page = 'https://github.com/jeanslack/Videomass/wiki'
        webbrowser.open(page)
        
    #------------------------------------------------------------------#
    def Issues(self, event):
        """Display Issues page on github"""
        page = 'https://github.com/jeanslack/Videomass/issues'
        webbrowser.open(page)
        
    #------------------------------------------------------------------#
    def Donation(self, event):
        """Display Issues page on github"""
        page = 'https://jeanslack.github.io/Videomass/donation.html'
        webbrowser.open(page)
        
    #------------------------------------------------------------------#
    def DocFFmpeg(self, event):
        """Display FFmpeg page documentation"""
        page = 'https://www.ffmpeg.org/documentation.html'
        webbrowser.open(page)
        
    #-------------------------------------------------------------------#
    def CheckNewReleases(self, event):
        """
        Check for new version releases of Videomass, useful for 
        users with Videomass installer on Windows and MacOs.
        """
        from videomass3.vdms_SYS.msg_info import current_release
        """
        FIXME : There are was some error regarding 
        [SSL: CERTIFICATE_VERIFY_FAILED]
        see:
        <https://stackoverflow.com/questions/27835619/urllib-and-ssl-
        certificate-verify-failed-error>
        <https://stackoverflow.com/questions/35569042/ssl-certificate-
        verify-failed-with-python3>
        """
        import ssl
        import urllib.request
        
        cr = current_release()
        
        #ssl._create_default_https_context = ssl._create_unverified_context

        try:
            context = ssl._create_unverified_context()
            f = urllib.request.urlopen('https://pypi.org/project/videomass/',
                                       context=context
                                       )
            #f = urllib.request.urlopen(
                            #'https://test.pypi.org/project/videomass/',
                            #context=context
                                       #)
            myfile = f.read().decode('UTF-8')
            page = myfile.strip().split()
            indx = ''
            for v in page:
                if 'class="package-header__name">' in v:
                    indx = page.index(v)

        except IOError as error:
            wx.MessageBox("%s" % error, "Videomass: ERROR", 
                          wx.ICON_ERROR, None
                          )
            return
        
        except urllib.error.HTTPError as error:
            wx.MessageBox("%s" % error, "Videomass: ERROR", 
                          wx.ICON_ERROR
                          )
            return
            
        if indx: 
            new_major, new_minor, new_micro =  page[indx+2].split('.')
            new_version = int('%s%s%s' %(new_major, new_minor, new_micro))
            this_major, this_minor, this_micro = cr[2].split('.')
            this_version = int('%s%s%s' %(this_major, this_minor, this_micro))
            
            if new_version > this_version:
                wx.MessageBox(_('A new version (v{0}) of Videomass is available'
                                '\nfrom <https://pypi.org/project/videomass/>') 
                            .format(page[indx+2]), "Videomass: Check new version", 
                                wx.ICON_INFORMATION, None
                                )
            else:
                wx.MessageBox(_('You are already using the latest version '
                                '(v{0}) of Videomass').format(cr[2]), 
                                "Videomass: Check new version", 
                                wx.ICON_INFORMATION, None
                                )
        else:
            wx.MessageBox(_('An error was found in the search for '
                            'the web page.\nSorry for this inconvenience.'),
                            "Videomass: Warning", wx.ICON_EXCLAMATION, None
                          )
            return
    
    #-------------------------------------------------------------------#
    def Info(self, event):
        """
        Display the program informations and developpers
        """
        infoprg.info(self, self.videomass_icon)
        
    #------------------------------------------------------------------#
        
###############################  BUILD THE TOOL BAR  ########################
    def videomass_tool_bar(self):
        """
        Makes and attaches the view toolsBtn bar
        """
        #--------- Properties
        self.toolbar = self.CreateToolBar(style=(wx.TB_HORZ_LAYOUT | wx.TB_TEXT))
        self.toolbar.SetToolBitmapSize((32,32))
        self.toolbar.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, ""))
        
        #-------- Import button
        icn_import = self.toolbar.AddTool(wx.ID_FILE3, _('Add Files'),
                                         wx.Bitmap(self.icon_import),
                                                )
        self.toolbar.EnableTool(wx.ID_FILE3, False)
        self.toolbar.AddSeparator()
        
        #-------- Switch at preset manager
        prs_mng = self.toolbar.AddTool(wx.ID_FILE5, _('Presets Manager'), 
                                    wx.Bitmap(self.icon_presets)
                                                )
        self.toolbar.EnableTool(wx.ID_FILE5, False)
        self.toolbar.AddSeparator()
        
        #-------- Switch at videomass
        switch_video = self.toolbar.AddTool(wx.ID_FILE6, 
                    _('Video Conversions'), wx.Bitmap(self.icon_switchvideomass)
                                                )
        self.toolbar.EnableTool(wx.ID_FILE6, False)
        self.toolbar.AddSeparator()

        #-------- Switch Advanced audio
        switch_audio = self.toolbar.AddTool(wx.ID_FILE7, 
                    _('Audio Conversions'),  wx.Bitmap(self.icon_headphones)
                                                )
        self.toolbar.EnableTool(wx.ID_FILE7, False)
        self.toolbar.AddSeparator()
        
        # ------- Run process button
        run_coding = self.toolbar.AddTool(wx.ID_OK, _('Start Encoding'), 
                                    wx.Bitmap(self.icon_process)
                                                )
        self.toolbar.EnableTool(wx.ID_OK, False)
        self.toolbar.AddSeparator()
        
        #------- help button
        help_contest = self.toolbar.AddTool(wx.ID_ANY, _('Help'), 
                                    wx.Bitmap(self.icon_help)
                                                )
        # finally, create it
        self.toolbar.Realize()
        
        #----------------- Tool Bar Binding (evt)-----------------------#
        self.Bind(wx.EVT_TOOL, self.File_import, icn_import)
        self.Bind(wx.EVT_TOOL, self.Preset_Mng, prs_mng)
        self.Bind(wx.EVT_TOOL, self.switch_video_conv, switch_video)
        self.Bind(wx.EVT_TOOL, self.switch_audio_conv, switch_audio)
        self.Bind(wx.EVT_TOOL, self.Run_Coding, run_coding)
        self.Bind(wx.EVT_TOOL, self.Helpme, help_contest)

    #--------------- Tool Bar Callback (event handler) -----------------#
    #------------------------------------------------------------------#
    def File_import(self, event):
        """
        Show files import panel.
        """
        self.PrstsPanel.Hide(), self.VconvPanel.Hide(), self.AconvPanel.Hide()
        self.DnD.Show()
        self.Layout()
        self.statusbar_msg(_('Add Files'), None)
        
        self.toolbar.EnableTool(wx.ID_FILE5, True)
        self.toolbar.EnableTool(wx.ID_FILE6, True)
        self.toolbar.EnableTool(wx.ID_FILE7, True)
        self.toolbar.EnableTool(wx.ID_FILE3, False)
        self.toolbar.EnableTool(wx.ID_OK, False)
        
        self.Setup_items_bar()
    #------------------------------------------------------------------#
    
    def Preset_Mng(self, event):
        """
        Show presets manager panel
        """
        self.panelshown = 'presets manager'
        self.file_sources = self.DnD.fileList[:]
        self.file_destin = self.DnD.file_dest
        
        self.DnD.Hide(), self.VconvPanel.Hide(), self.AconvPanel.Hide()
        self.PrstsPanel.Show(), self.Layout()

        self.statusbar_msg(_('Presets Manager'), None)
        self.Setup_items_bar()
    
        self.PrstsPanel.file_destin = self.file_destin
        if self.file_sources != self.PrstsPanel.file_sources:
            self.PrstsPanel.file_sources = self.file_sources
            self.duration = self.DnD.duration
    #------------------------------------------------------------------#
    def switch_video_conv(self, event):
        """
        Show Video converter panel
        """
        self.panelshown = 'video conversions'
        self.file_sources = self.DnD.fileList[:]
        self.file_destin = self.DnD.file_dest
        
        self.DnD.Hide(), self.PrstsPanel.Hide(), self.AconvPanel.Hide()
        self.VconvPanel.Show(), self.Layout()
        
        self.statusbar_msg(_('Video Conversions'), None)
        self.Setup_items_bar()

        self.VconvPanel.file_destin = self.file_destin
        if self.file_sources != self.VconvPanel.file_sources:
            self.VconvPanel.file_sources = self.file_sources
            self.VconvPanel.normalize_default()
            #self.VconvPanel.audio_default()
            self.duration = self.DnD.duration
    #------------------------------------------------------------------#
    #def switch_audio_conv(self, event):
    def switch_audio_conv(self, event):
        """
        Show Audio converter panel
        """
        self.panelshown = 'audio conversions'
        self.file_sources = self.DnD.fileList[:]
        self.file_destin = self.DnD.file_dest

        self.DnD.Hide(), self.PrstsPanel.Hide(), self.VconvPanel.Hide()
        self.AconvPanel.Show(), self.Layout()
        
        self.statusbar_msg(_('Audio Conversions'), None)
        self.Setup_items_bar()

        self.AconvPanel.file_destin = self.file_destin
        if self.file_sources != self.AconvPanel.file_sources:
            self.AconvPanel.file_sources = self.file_sources
            self.AconvPanel.normalization_disabled()
            self.duration = self.DnD.duration
            
    #------------------------------------------------------------------#
    def switch_Process(self, *varargs):
        """
        Show a panel for processing task only. 
        This is a panel that should not be instantiated at the beginning 
        of the main frame (as others) because otherwise it would immediately
        start running.
        """
        duration = self.DnD.duration[:] # the streams duration list
        
        if self.showpanelbar.IsChecked():
            self.btnpanel.Hide()# hide buttons bar if the user has shown it:

        IO_tools.process(self, varargs, 
                         self.panelshown, 
                         duration,
                         self.time_seq,
                         )
        #make the positioning:
        self.DnDsizer.Add(self.ProcessPanel, 1, wx.EXPAND|wx.ALL, 0)
        #Hide all others panels:
        self.DnD.Hide(), self.PrstsPanel.Hide(), self.VconvPanel.Hide()
        self.AconvPanel.Hide()
        #Show the panel:
        self.ProcessPanel.Show()
        self.Layout()
        self.SetTitle(_('..Start Encoding - Videomass'))

        self.Setup_items_bar()# call set default layout method
    #------------------------------------------------------------------#
    def Run_Coding(self, event):
        """
        By clicking the Start encoding button on the main frame, calls 
        the on_ok method of the corresponding panel shown, which calls 
        the 'switch_Process' method above.
        """
        if self.PrstsPanel.IsShown():
            self.PrstsPanel.on_ok()
        elif self.VconvPanel.IsShown():
            self.VconvPanel.on_ok()
        elif self.AconvPanel.IsShown():
            self.AconvPanel.on_ok()
            
    #------------------------------------------------------------------#
    def panelShown(self, panelshown):
        """
        When clicking 'close button' of the processing panel
        (see switch_Process method above), Retrieval at previous 
        panel showing and re-enables the functions provided by 
        the menu bar.
        """
        if panelshown == 'presets manager':
            self.ProcessPanel.Hide()
            self.Preset_Mng(self)
        elif panelshown == 'video conversions':
            self.ProcessPanel.Hide()
            self.switch_video_conv(self)
        elif panelshown == 'audio conversions':
            self.ProcessPanel.Hide()
            self.switch_audio_conv(self)
        # Enable all top menu bar:
        [self.menuBar.EnableTop(x, True) for x in range(0,3)]
        self.SetTitle("Videomass")# set the appropriate title
        # show buttons bar if the user has shown it:
        if self.showpanelbar.IsChecked():
            self.btnpanel.Show()
            self.Layout()
        
        
        



# -*- coding: UTF-8 -*-

#########################################################
# Name: presets_addnew.py
# Porpose: profile storing and profile editing dialog
# Compatibility: Python3, wxPython Phoenix
# Author: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2019 Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Rev (04) December 28 2018
#########################################################

# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################

import wx
import os
import string
from videomass3.vdms_IO.presets_manager_properties import delete_profiles
import webbrowser

# setting the path to the configuration directory:
get = wx.GetApp()
DIRconf = os.path.join(get.DIRconf, 'vdms')

class MemPresets(wx.Dialog):
    """
    Class for show dialog and store or edit single profiles of 
    the selected preset .
    """
    def __init__(self, parent, arg, full_pathname, filename, array, title):
        """
        arg: argument for evaluate if this dialog is used for add new
        profile or edit a existing profiles. Then you can pass three type 
        of strings: 
        arg = 'newprofile'  
        arg = 'edit'
        arg = 'addprofile'
        """
        wx.Dialog.__init__(self, parent, -1, title, style=wx.DEFAULT_DIALOG_STYLE)
        
        self.path_xml = full_pathname
        self.filename = filename
        self.arg = arg # arg é solo un parametro di valutazione (edit o newprofile).
        self.array = array
        
        self.txt_name = wx.TextCtrl(self, wx.ID_ANY, "", 
                                    style=wx.TE_PROCESS_ENTER
                                    )
        siz1_staticbox = wx.StaticBox(self, wx.ID_ANY, _("Profile Name"))
        self.txt_descript = wx.TextCtrl(self, wx.ID_ANY, "", 
                                        style=wx.TE_PROCESS_ENTER
                                        )
        siz2_staticbox = wx.StaticBox(self, wx.ID_ANY, _("Description"))
        self.txt_cmd = wx.TextCtrl(self, wx.ID_ANY, "", 
                                   style=wx.TE_PROCESS_ENTER | wx.TE_MULTILINE)
        siz3_staticbox = wx.StaticBox(self, wx.ID_ANY, 
                                      (_("Command Line Parameters "
                                   "Do not use `-i` or output filename)"))
                                      )
        self.txt_supp = wx.TextCtrl(self, wx.ID_ANY, "", 
                                    style=wx.TE_PROCESS_ENTER
                                    )
        siz4_supp = wx.StaticBox(self, wx.ID_ANY, (_("Files supported in the "
                                        "import (do not include the `.`)"))
                                                    )
        self.txt_ext = wx.TextCtrl(self, wx.ID_ANY, "", 
                                   style=wx.TE_PROCESS_ENTER
                                   )
        siz4_ext = wx.StaticBox(self, wx.ID_ANY, (_("Output format extension "
                                                  "(do not include the `.`)"))
                                                  )
        btn_help = wx.Button(self, wx.ID_HELP, "")
        btn_canc = wx.Button(self, wx.ID_CANCEL, "")
        btn_save = wx.Button(self, wx.ID_OK, _("Save.."))

        #----------------------Set Properties----------------------#
        self.txt_name.SetMinSize((150, -1))
        self.txt_descript.SetMinSize((300, -1))
        self.txt_cmd.SetMinSize((350, 60))
        self.txt_supp.SetMinSize((300, -1))
        self.txt_ext.SetMinSize((150, -1))

        self.txt_name.SetToolTip(_("Assign a short name to the profile. "
                                        ))
        self.txt_descript.SetToolTip(_("Assign a long description to "
                                           "the profile.")
                                           )
        self.txt_cmd.SetToolTip(_("Do not use `-i` or output file, please."
                                        ))
        self.txt_supp.SetToolTip(_("You can limit the type "
                            "of files formats imported by specifying in a box "
                            "a format or multiple formats separated by a space "
                            "(a list of extensions without the dot)."
                            "Leave blank to involve any type of file to import " 
                            "inbound."))
        self.txt_ext.SetToolTip(_("Write here the output format extension"))
        
        #----------------------Build layout----------------------#
        grd_s1 = wx.FlexGridSizer(4, 1, 0, 0)
        boxSiz = wx.BoxSizer(wx.VERTICAL)
        grdexit = wx.GridSizer(1, 2, 0, 0)
        grd_s4 = wx.GridSizer(1, 2, 0, 0)
        siz4_ext.Lower()
        s4_ext = wx.StaticBoxSizer(siz4_ext, wx.VERTICAL)
        siz4_supp.Lower()
        s4_f_supp = wx.StaticBoxSizer(siz4_supp, wx.VERTICAL)
        siz3_staticbox.Lower()
        siz3 = wx.StaticBoxSizer(siz3_staticbox, wx.VERTICAL)
        grd_s2 = wx.GridSizer(1, 2, 0, 0)
        siz2_staticbox.Lower()
        siz2 = wx.StaticBoxSizer(siz2_staticbox, wx.VERTICAL)
        siz1_staticbox.Lower()
        siz1 = wx.StaticBoxSizer(siz1_staticbox, wx.VERTICAL)
        siz1.Add(self.txt_name, 0, wx.ALL, 15)
        grd_s2.Add(siz1, 1, wx.ALL | wx.EXPAND, 15)
        siz2.Add(self.txt_descript, 0, wx.ALL, 15)
        grd_s2.Add(siz2, 1, wx.ALL | wx.EXPAND, 15)
        grd_s1.Add(grd_s2, 1, wx.EXPAND, 0)
        siz3.Add(self.txt_cmd, 0, wx.ALL|wx.EXPAND, 15)
        grd_s1.Add(siz3, 1, wx.ALL | wx.EXPAND, 15)
        s4_f_supp.Add(self.txt_supp, 0, wx.ALL, 15)
        grd_s4.Add(s4_f_supp, 1, wx.ALL | wx.EXPAND, 15)
        s4_ext.Add(self.txt_ext, 0, wx.ALL, 15)
        grd_s4.Add(s4_ext, 1, wx.ALL | wx.EXPAND, 15)
        grd_s1.Add(grd_s4, 1, wx.EXPAND, 0)
        grdBtn =  wx.GridSizer(1, 2, 0, 0)
        grdhelp = wx.GridSizer(1, 1, 0, 0)
        grdhelp.Add(btn_help, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        grdBtn.Add(grdhelp)
        grdexit.Add(btn_canc, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        grdexit.Add(btn_save, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        grdBtn.Add(grdexit, flag=wx.ALL|wx.ALIGN_RIGHT|wx.RIGHT, border=0)
        
        boxSiz.Add(grdBtn,1, wx.ALL | wx.EXPAND, 5)
        grd_s1.Add(boxSiz, 1, wx.ALL | wx.EXPAND, 5)
        self.SetSizer(grd_s1)
        grd_s1.Fit(self)
        self.Layout()

        #----------------------Binder (EVT)----------------------#
        self.Bind(wx.EVT_BUTTON, self.on_close, btn_canc)
        self.Bind(wx.EVT_BUTTON, self.on_help, btn_help)
        self.Bind(wx.EVT_BUTTON, self.on_apply, btn_save)
        
        #-------------------Binder (EVT) End --------------------#
        if arg == 'edit':
            self.change() # passo alla modifica del profilo, altrimenti
                        # vado avanti per memorizzarne di nuovi
        elif arg == 'addprofile':
            self.txt_cmd.AppendText(self.array[0]) # command or param
            if self.array[1] == '-c:v copy':
                self.txt_ext.AppendText('Not set')
            else:
                self.txt_ext.AppendText(self.array[1]) # extension
        
                        
    def change(self):
        """
        Copio gli elementi della lista array sui relativi campi di testo.
        questa funzione viene chiamata solo se si modificano i profili
        """
        self.txt_name.AppendText(self.array[0]) # name
        self.txt_descript.AppendText(self.array[1]) # descript
        self.txt_cmd.AppendText(self.array[2]) # command or param
        self.txt_supp.AppendText(self.array[3]) # file supportted
        self.txt_ext.AppendText(self.array[4]) # extension
    
#---------------------Callback (event handler)----------------------#
    
    def on_help(self, event):
        """
        """
        page = ('https://jeanslack.github.io/Videomass/Pages/Main_Toolbar/'
                'PresetsManager_Panel/Profiles_management.html')
        webbrowser.open(page)
    #------------------------------------------------------------------#
    def on_close(self, event):
        #self.Destroy()
        event.Skip()
    #------------------------------------------------------------------#
    def on_apply(self, event):
        
        nameprofile = self.txt_name.GetValue()
        descriptprofile = self.txt_descript.GetValue()
        paramprofile = self.txt_cmd.GetValue()
        file_support = self.txt_supp.GetValue()
        extprofile = self.txt_ext.GetValue() 
        
        if file_support in string.whitespace:
            wildcard = " "
        
        else:
            wildcard = file_support.strip()
            
        with open('%s' % (self.path_xml),'r') as readxml: # leggo il file
            rlist = readxml.readlines() # metto in lista il contenuto
        raw_list = ''.join(rlist)
        names = 'name="%s"' % (nameprofile)
        
        if nameprofile == '' or descriptprofile == '' or\
                                paramprofile == '' or extprofile ==  '':
                                    
            wx.MessageBox(_("Incomplete profile assignments"),
                             "Videomass ", wx.ICON_INFORMATION, self)
            return
                                            
        elif names in raw_list and self.arg == 'newprofile': # if exist name
                
            wx.MessageBox(_("Profile already stored with the same name"), 
                          "Videomass ", wx.ICON_INFORMATION, self)
            return

        if self.arg == 'edit':
            # call module-function in os_processing and pass list
            delete_profiles(self.array, self.filename, DIRconf)
            # riapro e leggo il file
            with open('%s' % (self.path_xml),'r') as open_xml: # leggo il file
                rlist = open_xml.readlines() # metto in lista il contenuto
            
        del rlist[len(rlist)-1] # cancello ultimo elemento
        #open('%s' % (self.path_xml),'w').writelines(rlist)
        with open('%s' % (self.path_xml), "w") as f:
            f.writelines(rlist)
        #rlist = open('%s' % (self.path_xml),'a') # mette in coda al file
        with open('%s' % (self.path_xml), "a") as rlist:

            model = """
    <label name="%s" type="%s">
        <parameters>%s</parameters>
        <filesupport>%s</filesupport>
        <extension>%s</extension>
    </label>
</presets>
""" % (nameprofile, descriptprofile, paramprofile, wildcard, extprofile)

            rlist.writelines(model)
        
        if self.arg == 'newprofile':
            wx.MessageBox(_("Successful storing!"))
            self.txt_name.SetValue(''), self.txt_descript.SetValue(''),
            self.txt_cmd.SetValue(''), self.txt_ext.SetValue('')
            self.txt_supp.SetValue('')
            
        elif self.arg == 'edit':
            wx.MessageBox(_("Successful changes!"))
            #self.Destroy() # con ID_OK e ID_CANCEL non serve
            
        elif self.arg == 'addprofile':
            wx.MessageBox(_("Successful storing!\n\n"
                          "You will find this profile in the 'User "
                          "Profiles'preset in the 'Presets Manager' panel.\n"
                          "Use the 'Reload presets list' on File menu to "
                          "update profile list."))
                
        event.Skip() 



# -*- coding: UTF-8 -*-
#
#########################################################
# Name: first_time_start.py
# Porpose: Automatize settings first time start
# Compatibility: Python3, wxPython Phoenix
# Author: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2019 Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Rev: Dec 28 2018, Aug. 28 2109
#########################################################

# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################

import wx
import os
from shutil import which
#---------------------------------------------------------------#

class FirstStart(wx.Dialog):
    """
    Shows a dialog wizard to locate FFmpeg executables
    """
    def __init__(self, img):
        """
        Set attribute with GetApp (see Videomass.py __init__)
        """
        get = wx.GetApp()
        self.FILEconf = get.FILEconf
        self.WORKdir = get.WORKdir
        self.OS = get.OS
        
        
        wx.Dialog.__init__(self, None, -1, style=wx.DEFAULT_DIALOG_STYLE)
        """constructor"""
        
        msg1 = (_(
            u"This wizard will attempt to automatically detect FFmpeg in\n"
            u"your system.\n\n"
            u"In addition, it allows you to manually set a custom path\n"
            u"to locate FFmpeg and its associated executables.\n\n"
            u"Also, Remember that you can always change these settings\n"
            u"later, through the Setup dialog.\n\n"
            u"- Press 'Auto-detection' to start the system search now."
            u"\n\n"
            u"- Press 'Browse..' to indicate yourself where FFmpeg is located.\n")
               )
        # widget:
        bitmap_drumsT = wx.StaticBitmap(self, wx.ID_ANY, wx.Bitmap(
                                        img,wx.BITMAP_TYPE_ANY))
        lab_welc1 = wx.StaticText(self, wx.ID_ANY, (
                                        _(u"Welcome to Videomass Wizard!")))
        lab_welc2 = wx.StaticText(self, wx.ID_ANY, (msg1))
        
        self.detectBtn = wx.Button(self, wx.ID_ANY, (_(u"Auto-detection")))

        self.browseBtn = wx.Button(self, wx.ID_ANY, (_(u"Browse..")))
        
        close_btn = wx.Button(self, wx.ID_EXIT, "")
        
        # properties
        self.SetTitle(_(u"Videomass: Wizard"))
        lab_welc1.SetFont(wx.Font(11, wx.DEFAULT, wx.NORMAL,wx.BOLD, 0, ""))
        # layout:
        sizer_base = wx.BoxSizer(wx.VERTICAL)
        grd_base = wx.FlexGridSizer(2, 1, 0, 0)
        grd_1 = wx.FlexGridSizer(1, 2, 0, 0)
        grd_ext = wx.FlexGridSizer(4, 1, 0, 0)
        grd_2 = wx.FlexGridSizer(3, 2, 0, 0)
        grd_base.Add(grd_1)
        grd_1.Add(bitmap_drumsT,0,wx.ALL, 10)
        grd_1.Add(grd_ext)
        grd_base.Add(grd_2)
        grd_ext.Add(lab_welc1,0,  wx.ALL, 10)
        grd_ext.Add(lab_welc2,0, wx.ALIGN_CENTER | wx.ALL, 10)
        grd_2.Add(self.detectBtn,0, wx.ALL, 15)

        grd_2.Add((260,0), 0, wx.ALL, 5)
        grd_2.Add(self.browseBtn,0, wx.ALL, 15)

        grd_2.Add((260,0), 0, wx.ALL, 5)
        grd_btn = wx.FlexGridSizer(1, 2, 0, 0)
        
        grd_btn.Add(close_btn,0, flag=wx.ALL, border=5)
        grd_2.Add((260,0), 0, wx.ALL, 15)
        grd_2.Add(grd_btn,0, flag=wx.ALL|wx.ALIGN_RIGHT|wx.RIGHT, border=10)
        #properties
        self.detectBtn.SetMinSize((200, -1))
        self.browseBtn.SetMinSize((200, -1))
        
        sizer_base.Add(grd_base)
        self.SetSizer(sizer_base)
        sizer_base.Fit(self)
        self.Layout()
        
        ######################## bindings #####################
        self.Bind(wx.EVT_BUTTON, self.On_close)
        self.Bind(wx.EVT_BUTTON, self.Detect, self.detectBtn)
        self.Bind(wx.EVT_BUTTON, self.Browse, self.browseBtn)
        self.Bind(wx.EVT_CLOSE, self.On_close) # controlla la chiusura (x)
        
    # EVENTS:
    #-------------------------------------------------------------------#
    def On_close(self, event):
        self.Destroy()
    #-------------------------------------------------------------------#
    def Browse(self, event):
        """
        The user find and import FFmpeg executables folder with
        ffmpeg, ffprobe, ffplay inside on Posix or ffmpeg.exe, ffprobe.exe, 
        ffplay.exe inside on Windows NT.
        """
        
        if self.OS == 'Windows':
            listFF = {'ffmpeg.exe':"",'ffprobe.exe':"",'ffplay.exe':""}
        else:
            listFF = {'ffmpeg':"",'ffprobe':"",'ffplay':""}
            
        dirdialog = wx.DirDialog(self, 
                _(u"Videomass: locate the ffmpeg folder"), "", 
                wx.DD_DEFAULT_STYLE | wx.DD_DIR_MUST_EXIST
                                )
            
        if dirdialog.ShowModal() == wx.ID_OK:
            path = u"%s" % dirdialog.GetPath()
            dirdialog.Destroy()
            
            filelist = []
            for ff in os.listdir(path):
                if ff in listFF.keys():
                    listFF[ff] = os.path.join("%s" % path, "%s" % ff)
                    
            error = False
            for key,val in listFF.items():
                if not val:
                    error = True
                    break
            if error:
                wx.MessageBox(_("File not found: '{0}'\n"
                                "'{1}' does not exist!\n\n"
                                "Need {2}\n\n"
                                "Please, choose a valid path.").format(
                                        os.path.join("%s" % path, "%s" % key),
                                        key,
                                        [k for k in listFF.keys()]), 
                                "Videomass: warning!",  wx.ICON_WARNING, self
                                )
                return
            
            self.completion([v for v in listFF.values()])

    #-------------------------------------------------------------------#
    def Detect(self, event):
        """
        Check for dependencies into your system (compatible with Linux, 
        MacOsX, Windows)
        [https://stackoverflow.com/questions/11210104/check-if-a-program-exists-
        from-a-python-script]
        Search the executable in the system, if fail stop the search, 
        otherwise write the executable pathname in the configuration file.
        """
        local = False
        if self.OS == 'Windows':
            biname = ['ffmpeg.exe','ffprobe.exe','ffplay.exe']
        else:
            biname = ['ffmpeg','ffprobe','ffplay']
            
        for required in biname:
            if which(required):
                print ("Check for: '%s' ..Ok" % required)
                no_which = False
            else:
                print ("Check for: '%s' ..Not Installed" % required)
                if self.OS == 'Darwin':
                    if os.path.isfile("/usr/local/bin/%s" % required):
                        local = True
                        no_which = False
                        break
                    else:
                        local = False
                        no_which = True
                        break
                elif self.OS == 'Windows':
                    no_which = True
                    break
        if no_which:
            for x in biname:
                if not os.path.isfile("%s/FFMPEG_BIN/bin/%s" %(self.WORKdir, x)):
                    noexists = True
                    break
                else:
                    noexists = False
            if noexists:
                wx.MessageBox(_("'%s' is not installed on the system.\n"
                          "Please, install it or set a custom path "
                          "using the 'Browse..' button.") 
                          % required, 'Videomass: Warning', 
                          wx.ICON_EXCLAMATION, self)
                return
            else:
                if wx.MessageBox(_("Videomass already seems to include "
                                   "FFmpeg and the executables associated "
                                   "with it.\n\n"
                                   "Are you sure you want to use them?"), 
                        _('Videomass: Please Confirm'),
                        wx.ICON_QUESTION |
                        wx.YES_NO, 
                        None) == wx.YES:
                    ffmpeg = "%s/FFMPEG_BIN/bin/%s" % (self.WORKdir, biname[0])
                    ffprobe = "%s/FFMPEG_BIN/bin/%s" % (self.WORKdir, biname[1])
                    ffplay = "%s/FFMPEG_BIN/bin/%s" % (self.WORKdir, biname[2])
                else:
                    return
        else:
            if local:
                ffmpeg = "/usr/local/bin/ffmpeg"
                ffprobe = "/usr/local/bin/ffprobe"
                ffplay = "/usr/local/bin/ffplay"
            else:
                ffmpeg = which(biname[0])
                ffprobe = which(biname[1])
                ffplay = which(biname[2])
        
        self.completion([ffmpeg,ffprobe,ffplay])
    #-------------------------------------------------------------------#
    
    def completion(self, FFmpeg):
        """
        Writes changes to the configuration file
        """
        ffmpeg = FFmpeg[0]
        ffprobe = FFmpeg[1]
        ffplay = FFmpeg[2]
        rowsNum = []#rows number list
        dic = {} # used for debug
        with open (self.FILEconf, 'r') as f:
            full_list = f.readlines()
        for a,b in enumerate(full_list):
            if not b.startswith('#'):
                if not b == '\n':
                    rowsNum.append(a)

        full_list[rowsNum[7]] = '%s\n' % ffmpeg
        full_list[rowsNum[9]] = '%s\n' % ffprobe
        full_list[rowsNum[11]] = '%s\n' % ffplay
        
        with open (self.FILEconf, 'w') as fileconf:
            for i in full_list:
                fileconf.write('%s' % i)
            
        wx.MessageBox(_("\nWizard completed successfully.\n"
                       "Restart Videomass now.\n\nThank You!"), 
                       _("That's all folks!"))   
        self.Destroy()
        



# -*- coding: UTF-8 -*-

#########################################################
# Name: while_playing.py
# Porpose: Show dialog box with shortcuts keyboard for FFplay
# Compatibility: Python3, wxPython4
# Author: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2019 Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Rev: Sept.02.2019
#########################################################

# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################

import wx

keys = (_("q, ESC\nf\np, SPC\nm\n9, 0\n/, *\na\nv\nt\nc\n"
          "w\ns\n\nleft/right\ndown/up\npage down/page up\n\n"
          "right mouse click\nleft mouse double-click"
          ))
explan = (_("Quiet.\nTogle full screen.\nPause.\nTogle mute.\n"
            "Decrease and increase volume respectively.\n"
            "Decrease and increase volume respectively.\n"
            "Cycle audio channel in the current program.\n"
            "Cycle video channel.\n"
            "Cycle subtitle channel in the current program.\n"
            "Cycle program.\nCycle video filters or show modes.\n"
            "Step to the next frame. Pause if the stream is not \n"#
            "already paused, step to the next video frame, and pause.\n"#
            "Seek backward/forward 10 seconds.\n"
            "Seek backward/forward 1 minute.\n"
            "Seek to the previous/next chapter. Or if there are no \n"#
            "chapters Seek backward/forward 10 minutes.\n"#
            "Seek to percentage in file corresponding to fraction of width.\n"
            "Toggle full screen."))

#------------------------------------------------------------------#    
class While_Playing(wx.Dialog):
    """
    Display a dialog box resizable with shortcuts keyboard
    useful when you use playback function with FFplay
   
    """
    def __init__(self, OS):
        wx.Dialog.__init__(self, None, style=wx.DEFAULT_DIALOG_STYLE)
        
        panel = wx.Panel(self, wx.ID_ANY, style=wx.TAB_TRAVERSAL)
        
        
        
        label1 = wx.StaticText(panel, wx.ID_ANY, keys)
        label2 = wx.StaticText(panel, wx.ID_ANY, explan)
        self.button_close = wx.Button(self, wx.ID_CLOSE, "")
        
        #----------------------Properties----------------------#
        self.SetTitle(_("Videomass: Shortcuts while playing"))
        label1.SetForegroundColour(wx.Colour('#008000'))
        label2.SetForegroundColour(wx.Colour('#959595'))
        panel.SetBackgroundColour(wx.Colour('#121212'))
        
        #if OS == 'Darwin':
            #label1.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL, wx.BOLD))
            #label2.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL, wx.NORMAL))

        #else:
            #label1.SetFont(wx.Font(8, wx.SWISS, wx.ITALIC, wx.BOLD))
            #label2.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.NORMAL))
        
        #---------------------- Layout ----------------------#
        s1 = wx.BoxSizer(wx.VERTICAL)
        gr_s1 = wx.FlexGridSizer(1, 2, 0, 0)
        gr_s1.Add(label1, 0, wx.ALL, 5)
        gr_s1.Add(label2, 0, wx.ALL, 5)
        btngrid = wx.FlexGridSizer(1,1,0,0)
        btngrid.Add(self.button_close, 0, wx.ALL, 5)
        panel.SetSizer(gr_s1)#
        #s1.Add(panel, 1, wx.ALL | wx.EXPAND, 5)
        s1.Add(panel, 1, )
        s1.Add(btngrid, flag=wx.ALL|wx.ALIGN_RIGHT|wx.RIGHT, border=5)
        
        #self.SetSizer(s1)
        #s1.Fit(self)
        #self.Layout()
        self.SetSizerAndFit(s1)
        
        # binding
        self.Bind(wx.EVT_BUTTON, self.on_close, self.button_close)
        self.Bind(wx.EVT_CLOSE, self.on_close) # controlla la chiusura (x)
        
    #--------------------------------------------------------------# 
    def on_close(self, event):
        '''
        destroy dialog by button and the X
        '''
        self.Destroy()
        #event.Skip()



# -*- coding: UTF-8 -*-

#########################################################
# File Name: preset_manager_properties.py
# Porpose: module for managing the properties of the preset manager panel
# Compatibility: Python3, wxPython Phoenix
# Author: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2019 Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Rev (12) December 28 2018
#########################################################

# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################

import wx
import os
from xml.dom.minidom import parseString

#------------------------------------------------------------------#
def supported_formats(array3, file_sources):
    """
    Verifica del supporto al formato dei file importati secondo
    le specifiche in array[3] in presets manager

    """
    supported = array3.split()# diventa una lista
    exclude = []# se viene riempita la usano i dialoghi MessageBox

    #-------------- SUPPORTED CONTROL
    if supported:
        for sources in file_sources:#itero su ogni file input
            basename = os.path.basename(sources)#nome file (con ext)
            base_ext = os.path.splitext(basename)[1]#estensione con dot
            ext = base_ext.replace('.','')
            if  ext not in supported:
                exclude.append(sources)
        if exclude:
            for x in exclude:
                file_sources.remove(x)
            if not file_sources:
                wx.MessageBox(_("The selected profile is not suitable to "
                                "convert the following file formats:\n\n%s\n\n") 
                              % ('\n'.join(exclude)), 
                              _("Videomass: Unsuitable format"), 
                              wx.ICON_WARNING | wx.OK, 
                              None
                              )
                return (False)
            
    return (file_sources)

########################################################################
# PARSINGS XML FILES AND FUNCTION FOR DELETING
########################################################################

def parser_xml(arg, src):
    """
    Used by presets_mng_panel.py to make parsing and read the xml data 
    on vdms files (the vdms files are xml files with extension '.vdms').
    The (arg) argument is one of the filename of the vdms file to parse. 
    The xml.dom.minidom it is used to parsing vdms files and return a value 
    consisting of dictionaries containing other dictionaries in this form:
    {name: {filesupport:?, descript:?, command:?, extens:?}, else: {etc etc}}
    
    """
    prst = os.path.join('%s' % src, '%s.vdms' % arg)
    with open(prst,'r') as fread:
        data = fread.read()

    parser = parseString(data) # fa il parsing del file xml ed esce: 
                            # <xml.dom.minidom.Document instance at 0x8141eec>
    dati = dict() 
    for presets in parser.getElementsByTagName("presets"):
        for preset in presets.getElementsByTagName("label"):
            name = preset.getAttribute("name")
            types = preset.getAttribute("type")
            parameters = None
            filesupport = None
            extension = None
            
            for presets in preset.getElementsByTagName("parameters"):
                for parameters in presets.childNodes:
                    params = parameters.data
                    
            for presets in preset.getElementsByTagName("filesupport"):
                for filesupport in presets.childNodes:
                    support = filesupport.data

            for presets in preset.getElementsByTagName("extension"):
                for extension in presets.childNodes:
                    ext = extension.data

            dati[name] = { "type": types, "parametri": params, 
                           "filesupport": support, "estensione": ext }
            
    #return dati[name]

    return dati
#------------------------------------------------------------------#

def delete_profiles(array, filename, DIRconf):
    """
    Funzione usata nel modulo presets_mng_panel.py per cancellare singole 
    voci (i profili) dei presets selezionati nella listcontrol.

    dati é il dizionario iterabile passato dal modulo parser_xml qui sopra

    array é solo una chiave del diz messa in lista e contiene cinque elementi: 
    name descr command  support ext 
    Array viene creata in presets_mng_panel.py nella 
    funzione def on_select()

    """
    
    dati = parser_xml(filename, DIRconf)
    dirconf = os.path.join('%s' % DIRconf, '%s.vdms' % filename)
    
    """
    Posso anche usare i dizionari al posto degli indici lista (sotto i cicli 
    for) ma ho deciso per gli indici lista, es: 
    
    #name_preset = array[0].encode("utf-8")
    #description = param["type"].encode("utf-8")
    #commands_ffmpeg = param["parametri"].encode("utf-8")  # not more used
    #file_support = param["filesupport"].encode("utf-8")   # not more used
    #file_extension = param["estensione"].encode("utf-8")  # not more used
    
    """
    param = dati[array[0]] # da il dizionario completo
    name_preset = array[0]
    description = array[1]

    #fxml = open('%s' % (dirconf),'r').readlines()
    with open('%s' % (dirconf),'r') as data:
        fxml = data.readlines()
    
    """
    Nella versione precedente, usavo il sistema con il metodo .index() ma 
    causava degli errori. Non trova title=fxml.index a volte (Vedi il 
    changelog) per errori di occorrenze.

    SOLUZIONE:
    title + 1 (oppure indx +1 -vedi sotto-) avanza sempre di uno rispetto a
    title, e sempre solo a quello. Questa soluzione dovrebbe impedire di 
    eliminare identiche occorrenze tipo <filesupport> </filesupport> nella 
    ricerca in fxml
    
    """
    title = '    <label name="%s" type="%s">\n' % (name_preset, description)
    
    for indx, row in enumerate(fxml): # mi da l'indx (come index)
                                      # ma anche la stringa in riga
        if title in row:
            
            for i in range(0,4): # ripete la stessa esecuzione per 4 volte
                del fxml[indx + 1]
            # il for sopra, mi serializza l'esecuzione di 'del' per 4 volte.
            # Fa la stessa cosa del codice fra i trattini qui sotto ma é più 
            # elegante:
            #----------------------------------------------#
            #del fxml [indx + 1] # label iniziale
            #del fxml [indx + 1] # command ffmpeg
            #del fxml [indx + 1] # file support
            #del fxml [indx + 1] # estensione
            #----------------------------------------------#

            if fxml [indx + 1] == "\n":
                del fxml [indx + 1] # lo spazio
                
            del fxml [indx] # per ultimo cancello il name e type
            
    # FIXME: c'é sempre una linea in più quando cancelli tutti i profili,
    #        questo non impedisce la corretta esecuzione ma influenza la
    #        formattazione del testo. L'occhio vuole la sua parte!

    with open('%s' % (dirconf),'w') as f:
        f.writelines(fxml)



# -*- coding: UTF-8 -*-

#########################################################
# Name: ctrl_run.py
# Porpose: Program boot data
# Compatibility: Python3
# Author: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2019 Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Rev: Dec. 28 2018, Aug.28 2019
#########################################################

# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################

import sys
import os
import shutil
import platform

# Set default variables
WORKdir = os.getcwd() # work current directory (where is Videomsass?)
USERName = os.path.expanduser('~') # /home/user (current user directory)
OS = platform.system()# What is the OS ??

# Establish the conventional paths on the different OS where 
# the videomass configuration directory will be stored:
if OS == 'Windows':
    bpath = "\\AppData\\Roaming\\videomass\\videomassWin32.conf"
    FILEconf = os.path.join(USERName + bpath)
    DIRconf = os.path.join(USERName + "\\AppData\\Roaming\\videomass")

elif OS == "Darwin":
    bpath = "/Library/Application Support/videomass/videomass.conf"
    FILEconf = os.path.join(USERName + bpath)
    DIRconf = os.path.join(USERName + os.path.dirname(bpath))
    
else: # Linux, FreeBsd, etc.
    bpath = "/.config/videomass/videomass.conf"
    FILEconf = os.path.join(USERName + bpath)
    DIRconf = os.path.join(USERName + "/.config/videomass")

#------------------------------------------------------------------------#

def parsing_fileconf():
    """
    Make a parsing of the configuration file and return
    object list with the current program settings data.
    
    """
    with open (FILEconf, 'r') as f:
        fconf = f.readlines()
    lst = [line.strip() for line in fconf if not line.startswith('#')]
    dataconf = [x for x in lst if x]# list without empties values
    if not dataconf:
        return 'corrupted'
    else:
        return dataconf
#------------------------------------------------------------------------#

def system_check():
    """
    assigning shared data paths and 
    checking the configuration folder  
    
    """
    #----------------------------------------------------------- #
    #### Set locale path and source data for restoring if needed #
    #------------------------------------------------------------#
    if os.path.isdir('%s/art' % WORKdir):
        #launch in local on any OS or as .exe and .app also
        localepath = 'locale'
        SRCpath = '%s/share' % WORKdir
        IS_LOCAL = True
        
    else: # Path system installation (usr, usr/local, ~/.local)
        if OS == 'Windows':
            #Installed with 'pip install videomass' command
            pythonpath = os.path.USERName(sys.executable)
            localepath = pythonpath + '\\share\\locale'
            SRCpath = pythonpath + '\\share\\videomass\\config'
            IS_LOCAL = False
            
        else:
            binarypath = shutil.which('videomass')
            if binarypath == '/usr/local/bin/videomass':
                #usually Linux,MacOs,Unix
                localepath = '/usr/local/share/locale'
                SRCpath = '/usr/local/share/videomass/config'
                IS_LOCAL = False
            elif binarypath == '/usr/bin/videomass':
                #usually Linux
                localepath = '/usr/share/locale'
                SRCpath = '/usr/share/videomass/config'
                IS_LOCAL = False
            else:
                #installed with 'pip install --user videomass' command
                import site
                userbase = site.getuserbase()
                localepath = userbase + '/share/locale'
                SRCpath = userbase + '/share/videomass/config'
                IS_LOCAL = False
                
    #--------------------------------------------#
    #### check videomass.conf and config. folder #
    #--------------------------------------------#
    copyerr = False
    existfileconf = True # file conf esiste (True) o non esiste (False)
    
    if os.path.exists(os.path.dirname(FILEconf)): # if exist conf. folder
        if os.path.isfile(FILEconf):
            DATAconf = parsing_fileconf() # fileconf data
            if DATAconf == 'corrupted':
                print ("The file configuration is corrupted! try to restore..")
                existfileconf = False
            if float(DATAconf[0]) != 1.4:
                existfileconf = False
        else:
            existfileconf = False
        
        if not existfileconf:
            try:
                if OS == ('Linux') or OS == ('Darwin'):
                    shutil.copyfile('%s/videomass.conf' % SRCpath, 
                                    FILEconf)
                elif OS == ('Windows'):
                    shutil.copyfile('%s/videomassWin32.conf' % SRCpath, 
                                    FILEconf)
                DATAconf = parsing_fileconf() # read again file conf
            except IOError:
                copyerr = True
                DATAconf = 'corrupted'
    else:
        try:
            shutil.copytree(SRCpath, DIRconf)
            DATAconf = parsing_fileconf() #  read again file conf
        except OSError:
            copyerr = True
            DATAconf = 'corrupted'
        except IOError:
            copyerr = True
            DATAconf = 'corrupted'

    return (OS, SRCpath, 
            copyerr, IS_LOCAL, 
            DATAconf, localepath,
            FILEconf, WORKdir, DIRconf)

#------------------------------------------------------------------------#



# -*- coding: UTF-8 -*-

#########################################################
# Name: check_bin.py
# Porpose: Gets the output to display the features of FFmpeg
# Compatibility: Python3 (Unix, Windows)
# Author: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2019 Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Rev: Aug.23 2019
#########################################################

# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################

import subprocess

#------------------------------------------------------#
#------------------------------------------------------#
def subp_win32(args):
    """
    Execute commands received by args parameter on Windows 
    OS only.
    This function uses 'Popen' to invoke the subprocess, since 
    on Windows it is necessary to use the STARTUPINFO class, 
    therefore the following constants are available only on 
    Windows.
    
    """
    cmd = [] 
    for opt in args:
        cmd.append(opt)
    cmd = ' '.join(cmd)
        
    try:
        startupinfo = subprocess.STARTUPINFO()
        startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
        p = subprocess.Popen(cmd, 
                             stdout=subprocess.PIPE, 
                             stderr=subprocess.STDOUT,
                             universal_newlines=True, # mod text
                             startupinfo=startupinfo,
                             )
        out =  p.communicate()
    
    except OSError as oserr:# if ffmpeg do not exist
        return('Not found', oserr)
        
    if p.returncode: # if returncode == 1
        return('Not found', out[0])
    
    return ('None', out[0])
#-----------------------------------------------------------#

def subp(args):
    """
    Execute commands received by args parameter on Unix-like 
    OS only.
    This function uses 'run' to invoke the subprocess, since 
    these are not advanced use cases.
    
    """
    cmd = [] 
    for opt in args:
        cmd.append(opt)
        
    try:
        p = subprocess.run(cmd, 
                           capture_output=True, 
                           universal_newlines=True
                           # mod text, otherwise must be used p.stdout.decode()
                           # for decode bytestring (b'')
                           )
        
    except FileNotFoundError as err:# if ffmpeg do not exist
        return('Not found', err)

    if p.returncode:# if has error on args
        err = p.stderr
        return('Not found', err)
    
    return ('None', p.stdout)
#-----------------------------------------------------------#

def ff_conf(ffmpeg_link, OS):
    """
    Receive output of the passed command to parse 
    configuration messages of FFmpeg
    
    ...Returns lists of:
        - info = [version number etc, release and building]
        - others = [building flags]
        - enable = [enabled features]
        - disable = [disable features]
    
    ...If errors returns 'Not found'
    
    """
    # grab generic informations:
    if OS == 'Windows':
        vers = subp_win32([ffmpeg_link, '-loglevel', 'error', '-version'])
    else:
        vers = subp([ffmpeg_link, '-loglevel', 'error', '-version'])
        
    if 'Not found' in vers[0]:
        return(vers[0], vers[1])

    info = []
    for v in vers[1].split('\n'):
        if 'ffmpeg version' in v:
            info.append(v.strip())
            
        if 'built with' in v:
            info.append(v.strip())
            
    # grab buildconf:
    if OS == 'Windows':
        build = subp_win32([ffmpeg_link, '-loglevel', 'error', '-buildconf'])
    else:
        build = subp([ffmpeg_link, '-loglevel', 'error', '-buildconf'])
    
    if 'Not found' in build[0]:
        return(build[0], build[1])

    conf = []
    for c in build[1].split('\n'):
        conf.append(c.strip())

    enable = []
    disable = []
    others = []

    for en in conf:
        if en.startswith('--enable'):
            enable.append(en.split('--enable-')[1])
            
        elif en.startswith('--disable'):
            disable.append(en.split('--disable-')[1])
            
        else:
            others.append(en)
            
    if 'configuration:' in others:
        others.remove('configuration:')
    
    return(info, others, enable, disable)
#-------------------------------------------------------------------#

def ff_formats(ffmpeg_link, OS):
    """
    Receive output of *ffmpeg -formats* command and return a 
    ditionary with the follow keys and values:
    
        * KEYS                  * VALUES 
        'Demuxing Supported' :  [list of (D)emuxing formats support]
        'Muxing Supported' :    [list of (M)uxing formats support]
        'Mux/Demux Supported' : [list of (D)emuxing(M)uxing formats support]
    
    """
    # grab buildconf:
    if OS == 'Windows':
        ret = subp_win32([ffmpeg_link, '-loglevel', 'error', '-formats'])
    else:
        ret = subp([ffmpeg_link, '-loglevel', 'error', '-formats'])
    
    if 'Not found' in ret[0]:
        return({ret[0]:ret[1], '':'', '':''})
    
    frmt = ret[1].split('\n')
    
    
    dic = {'Demuxing Supported':[], 
           'Muxing Supported':[], 
           'Mux/Demux Supported':[]}
    
    for f in frmt:
        if f.strip().startswith('D '):
            dic['Demuxing Supported'].append(f.replace('D', '', 1).strip())
        elif f.strip().startswith('E '):
            dic['Muxing Supported'].append(f.replace('E', '', 1).strip())
        elif f.strip().startswith('DE '):
            dic['Mux/Demux Supported'].append(f.replace('DE', '', 1).strip())
            
    return(dic)
#-------------------------------------------------------------------#

def ff_codecs(ffmpeg_link, type_opt, OS):
    """
    Receive output of *ffmpeg -encoders* or *ffmpeg -decoders*
    command and return a ditionary with the follow keys and values:
    
        * KEYS                  * VALUES 
        'Video' :    [list of V.....   name    descrition]
        'Audio' :    [list of A.....   name    descrition]
        'Subtitle' : [list of S.....   name    descrition]
    
    Accept two arguments: the call to FFmpeg and the type of 
    option used to obtain data on the encoding and decoding 
    capabilities of FFmpeg. The options to pass as arguments 
    are the following:
    
    -encoders
    -decoders
    
    """
    # grab encoders or decoders output:
    if OS == 'Windows':
        ret = subp_win32([ffmpeg_link, '-loglevel', 'error', type_opt])
    else:
        ret = subp([ffmpeg_link, '-loglevel', 'error', type_opt])
    
    if 'Not found' in ret[0]:
        return({ret[0], ret[1]})
    
    codecs = ret[1].split('\n')
    
    dic = {'Video':[], 'Audio':[], 'Subtitle':[]}
    
    for f in codecs:
        if f.strip().startswith('V'):
            if not 'V..... = Video' in f:
                dic['Video'].append(f.strip())
                
        elif f.strip().startswith('A'):
            if not 'A..... = Audio' in f:
                dic['Audio'].append(f.strip())
                
        elif f.strip().startswith('S'):
            if not 'S..... = Subtitle' in f:
                dic['Subtitle'].append(f.strip())
            
    return(dic)
#-------------------------------------------------------------------#

def ff_topics(ffmpeg_link, topic, OS):
    """
    Get output of the options help command of FFmpeg. 
    Note that the 'topic' parameter is always a list.
    
    """
    # get output:
    arr = [ffmpeg_link, '-loglevel', 'error'] + topic 
    if OS == 'Windows':
        ret = subp_win32(arr)
    else:
        ret = subp(arr)
    
    if 'Not found' in ret[0]:
        return(ret[0], ret[1])
    
    return ('None', ret[1])



# -*- coding: UTF-8 -*-

#########################################################
# Name: ffmpeg_conf.py
# Porpose: Dialog to show the build configuration of the FFmpeg
# Compatibility: Python3, wxPython4
# Author: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2019 Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Rev: Aug.18 2019
#########################################################

# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################

import wx
import os
from shutil import which

class Checkconf(wx.Dialog):
    """
    View the features of the build configuration of 
    FFmpeg on different notebook panels
    
    """
    def __init__(self, out, ffmpeg_link, ffprobe_link, ffplay_link, OS):
        """
        with 'None' not depend from parent:
        wx.Dialog.__init__(self, None, style=wx.DEFAULT_DIALOG_STYLE)
        
        With parent, -1:
        wx.Dialog.__init__(self, parent, -1, style=wx.DEFAULT_DIALOG_STYLE)
        if close videomass also close parent window:
        
        """
        wx.Dialog.__init__(self, None, style=wx.DEFAULT_DIALOG_STYLE)
        notebook_1 = wx.Notebook(self, wx.ID_ANY)
        notebook_1_pane_1 = wx.Panel(notebook_1, wx.ID_ANY,)
        txtinfo = wx.StaticText(notebook_1_pane_1, wx.ID_ANY,)
        notebook_1_pane_2 = wx.Panel(notebook_1, wx.ID_ANY)
        others_opt = wx.ListCtrl(notebook_1_pane_2, wx.ID_ANY, 
                                    style=wx.LC_REPORT | 
                                    wx.SUNKEN_BORDER
                                    )
        notebook_1_pane_3 = wx.Panel(notebook_1, wx.ID_ANY)
        enable_opt = wx.ListCtrl(notebook_1_pane_3, wx.ID_ANY, 
                                     style=wx.LC_REPORT | 
                                     wx.SUNKEN_BORDER
                                     )
        notebook_1_pane_4 = wx.Panel(notebook_1, wx.ID_ANY)
        
        disabled_opt = wx.ListCtrl(notebook_1_pane_4, wx.ID_ANY, 
                                       style=wx.LC_REPORT | 
                                       wx.SUNKEN_BORDER
                                       )
        #button_help = wx.Button(self, wx.ID_HELP, "")
        button_close = wx.Button(self, wx.ID_CLOSE, "")
        
        #----------------------Properties----------------------#
        self.SetTitle(_("Videomass: FFmpeg specifications"))
        others_opt.SetMinSize((700, 400))
        others_opt.InsertColumn(0, _('flags'), width=300)
        others_opt.InsertColumn(1, _('options'), width=450)
        #others_opt.SetBackgroundColour(wx.Colour(217, 255, 255))
        enable_opt.SetMinSize((700, 400))
        enable_opt.InsertColumn(0, _('status'), width=300)
        enable_opt.InsertColumn(1, _('options'), width=450)
        #enable_opt.SetBackgroundColour(wx.Colour(217, 255, 255))
        disabled_opt.SetMinSize((700, 400))
        disabled_opt.InsertColumn(0, _('status'), width=300)
        disabled_opt.InsertColumn(1, _('options'), width=450)
        #disabled_opt.SetBackgroundColour(wx.Colour(217, 255, 255))
        
        if OS == 'Darwin':
            txtinfo.SetFont(wx.Font(12, wx.SWISS, wx.NORMAL, wx.NORMAL))
            others_opt.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL, wx.NORMAL))
            enable_opt.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL, wx.NORMAL))
            disabled_opt.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL, wx.NORMAL))
        else:
            txtinfo.SetFont(wx.Font(9, wx.SWISS, wx.NORMAL, wx.NORMAL))
            others_opt.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL))
            enable_opt.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL))
            disabled_opt.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL))
        
        #----------------------Layout--------------------------#
        Base = wx.BoxSizer(wx.VERTICAL)

        grid_buttons = wx.FlexGridSizer(1, 1, 0, 0)
        sizer_tab1 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_tab2 = wx.BoxSizer(wx.VERTICAL)
        sizer_tab3 = wx.BoxSizer(wx.VERTICAL)
        sizer_tab4 = wx.BoxSizer(wx.VERTICAL)

        grid_tab1 = wx.FlexGridSizer(1, 1, 10, 10)
        sizer_tab1.Add(grid_tab1)
        grid_tab1.Add(txtinfo, 1, wx.ALL , 5)
        notebook_1_pane_1.SetSizer(sizer_tab1)
        
        sizer_tab2.Add(others_opt, 1, wx.ALL | wx.EXPAND, 5)
        notebook_1_pane_2.SetSizer(sizer_tab2)
        
        sizer_tab3.Add(enable_opt, 1, wx.ALL | wx.EXPAND, 5)
        notebook_1_pane_3.SetSizer(sizer_tab3)
        
        sizer_tab4.Add(disabled_opt, 1, wx.ALL | wx.EXPAND, 5)
        notebook_1_pane_4.SetSizer(sizer_tab4)
        
        notebook_1.AddPage(notebook_1_pane_1, (_("Informations")))
        notebook_1.AddPage(notebook_1_pane_2, (_("System options")))
        notebook_1.AddPage(notebook_1_pane_3, (_("Options enabled")))
        notebook_1.AddPage(notebook_1_pane_4, (_("Options disabled")))

        Base.Add(notebook_1, 1, wx.ALL|wx.EXPAND, 5)####
        grid_buttons.Add(button_close, 0, wx.ALL, 5)

        Base.Add(grid_buttons, flag=wx.ALIGN_RIGHT|wx.RIGHT, border=0)####

        self.SetSizer(Base)
        Base.Fit(self)
        self.Layout()
        
        # create lists by out:
        info, others, enable, disable = out
        #---------
        if OS == 'Windows':
            biname = ['ffmpeg.exe','ffprobe.exe','ffplay.exe']
        else:
            biname = ['ffmpeg','ffprobe','ffplay']
        #---------
        if which(biname[0]):
            ffmpeg = _("FFmpeg   ...installed")
        else:
            if os.path.exists(ffmpeg_link):
                ffmpeg = _("FFmpeg   ...was imported locally")
            
        if which(biname[1]):
            ffprobe = _("FFprobe   ...installed")
        else:
            if os.path.exists(ffprobe_link):
                ffprobe = _("FFprobe   ...was imported locally")
            else:
                ffprobe = _("FFprobe   ...not found !")
                
        if which(biname[2]):
            ffplay = _("FFplay   ...installed")
        else:
            if os.path.exists(ffplay_link):
                ffplay = _("FFplay   ...was imported locally")
            else:
                ffplay = _("FFplay   ...not found !")
        
        #### populate txtinfo TextCtrl output:
        txtinfo.SetLabel("\n\n\n\n"
                         "              %s\n"
                         "              %s\n"
                         "\n\n\n"
                         "             - %s\n"
                         "             - %s\n"
                         "             - %s\n" % (info[0].strip(),
                                                  info[1].strip(),
                                                  ffmpeg,
                                                  ffprobe,
                                                  ffplay,
                                                  ))
        #### populate others_opt listctrl output:
        index = 0 
        if not others:
            print ('No others option found')
        else:
            others_opt.InsertItem(index, _('Specific compilation options'))
            others_opt.SetItemBackgroundColour(index, "CORAL")
            n = len(others)
            for a in range(n):
                if '=' in others[a]:
                    (key, value) = others[a].strip().split('=')
                    #(key, value) = others[a][0].strip().split('=')
                    num_items = others_opt.GetItemCount()
                    index +=1
                    others_opt.InsertItem(index, key)
                    others_opt.SetItem(index, 1, value)
                
        #### populate enable_opt listctrl output:
        index = 0
        if not enable:
            print ('No options enabled')
        else:
            enable_opt.InsertItem(index, _('ENABLED:'))
            enable_opt.SetItemBackgroundColour(index, "GREEN")
            n = len(enable)
            for a in range(n):
                (key, value) = _('Enabled'), enable[a]
                num_items = enable_opt.GetItemCount()
                index +=1
                enable_opt.InsertItem(index, key)
                enable_opt.SetItem(index, 1, value)
        
        #### populate disabled_opt listctrl output:
        index = 0 
        if not disable:
            print ('No options disabled')
        else:
            disabled_opt.InsertItem(index, _('DISABLED:'))
            disabled_opt.SetItemBackgroundColour(index, "RED")
            n = len(disable)
            for a in range(n):
                (key, value) = _('Disabled'), disable[a]
                num_items = disabled_opt.GetItemCount()
                index +=1
                disabled_opt.InsertItem(index, key)
                disabled_opt.SetItem(index, 1, value)
                    
        #----------------------Binding (EVT)----------------------#
        self.Bind(wx.EVT_BUTTON, self.on_close, button_close)
        self.Bind(wx.EVT_CLOSE, self.on_close) # controlla la chiusura (x)
        #self.Bind(wx.EVT_BUTTON, self.on_help, button_help)

    #----------------------Event handler (callback)----------------------#
    def on_close(self, event):
        self.Destroy()
        #event.Skip()

    #-------------------------------------------------------------------#



# -*- coding: UTF-8 -*-

#########################################################
# Name: os_interaction.py
# Porpose: module with standard tools for OS
# Compatibility: Python3, wxPython Phoenix
# Author: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2019 Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Rev December 28 2018
#########################################################

# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################

"""
The module contains some useful function for copying, moving, deleting 
and renaming files and folders.

"""
import shutil
import os
import glob


def copy_restore(src, dest):
    """
    function for restore file. File name is owner choice and can be an preset
    or not. If file exist overwrite it.
    """
    shutil.copyfile(src, '%s' % (dest))

#------------------------------------------------------------------#
def copy_backup(src, dest):
    """
    function for backup file. File name is owner choice
    """
    shutil.copyfile('%s' % (src), dest)

#------------------------------------------------------------------#
def makedir_move(ext, name_dir):
    """
    this function is for make directory and move-in file (ext, name_dir: 
    extension, directory name)
    """
    try: # if exist dir not exit OSError, go...
        os.mkdir("%s" % (name_dir))
    except: OSError
    move_on(ext, name_dir)

#------------------------------------------------------------------#
def rename_move_indir(src, ext, name_dir):
    """
    this function includes "makedir_move(ext, name_dir)" function but cycling 
    files renames groups
    """
    for path in os.listdir(os.getcwd()):
        nuovoNome = path.replace("%s" % (src), "%s" % (ext))
        os.rename(path, nuovoNome)
    try:
        os.mkdir("%s" % (name_dir))
    except: OSError
    
    move_on(ext, name_dir)

#------------------------------------------------------------------#
def rename_file(src, ext):
    """
    Cycling for file rename groups only
    """
    for path in os.listdir(os.getcwd()):
        nuovoNome = path.replace("%s" % (src), "%s" % (ext))
        os.rename(path,nuovoNome)

#------------------------------------------------------------------#
def move_on(ext, name_dir):
    """
    Cycling on name extension file and move-on in other directory
    """
    files = glob.glob("*%s" % (ext))
    for sposta in files:
        #shutil.move(sposta, '%s' % (name_dir))
        print ('%s   %s' % (sposta,name_dir))
        
#------------------------------------------------------------------#
def copy_on(ext, name_dir, path_confdir):
    """
    Cycling on path and file extension name for copy files in other directory
    ARGUMENTS:
    ext: files extension with no dot
    name_dir: path name with no basename
    """
    files = glob.glob("%s/*.%s" % (name_dir, ext))
    
    for copia in files:
        shutil.copy(copia, '%s' % (path_confdir))

#------------------------------------------------------------------#
def delete(ext):
    """
    function for file group delete with same extension
    """
    files = glob.glob("*%s" % (ext))
    for rimuovi in files:
        os.remove(rimuovi)

#------------------------------------------------------------------#
def delete_noempty_dir(path):
    """
    Delete a entire directory no empty
    """
    shutil.rmtree(path)

#------------------------------------------------------------------#
#def exist_file(inputfile):
    #"""
    #Control if exist an file name
    
    #"""
    #os.chdir(PWD)
    #file_exist =  os.path.isfile(os.path.join(PWD,'%s' % inputfile))
    #if file_exist is False:
        #call('clear', shell = True) 
        #print ("Nella directory: '%s'\nNessun file da processare che abbia "
            #"questo nome:\033[0;1m %s\033[0m" % (PWD, inputfile)
            #)
        #sys.exit("...Error\n")
